@@ -1,14 +1,9 @@
-#include "dosbox.h"
 
-#include "control.h"
 #include "menu.h"
 #include "mouse.h"
-#include "render.h"
 #include "video.h"
 
-#include "cross.h"
 #include "SDL.h"
-#include "SDL_video.h"
 
 #ifdef __WIN32__
 #include "SDL_syswm.h"
@@ -23,6 +18,7 @@ enum SCREEN_TYPES {
 #if C_DIRECT3D
     ,SCREEN_DIRECT3D
 #endif
+    ,SCREEN_TTF
 };
 
 enum AUTOLOCK_FEEDBACK
@@ -123,6 +119,7 @@ struct SDL_Block {
         bool prevent_fullscreen = false;
         bool lazy_fullscreen_req = false;
         bool doublebuf = false;
+        bool isperfect = false;
         SCREEN_TYPES type = (SCREEN_TYPES)0;
         SCREEN_TYPES want_type = (SCREEN_TYPES)0;
     } desktop;
@@ -142,12 +139,12 @@ struct SDL_Block {
     SDL_Window * window = NULL;
     SDL_Renderer * renderer = NULL;
     const char * rendererDriver = NULL;
-    int displayNumber = 0;
     struct {
         SDL_Texture * texture = NULL;
         SDL_PixelFormat * pixelFormat = NULL;
     } texture;
 #endif
+    int displayNumber = 0;
     SDL_cond *cond = NULL;
     struct {
         bool autolock = false;
@@ -167,6 +164,7 @@ struct SDL_Block {
     bool using_windib = false;
     // Time when sdl regains focus (alt-tab) in windowed mode
     uint32_t focus_ticks = 0;
+    uint32_t ime_ticks;
 #endif
     // state of alt-keys for certain special handlings
     uint16_t laltstate = 0, raltstate = 0;
@@ -223,4 +221,9 @@ SDL_Window* GFX_SetSDLWindowMode(uint16_t width, uint16_t height, SCREEN_TYPES s
 void SDL_GL_SwapBuffers(void);
 #endif
 
+bool TTF_using(void);
+#ifdef USE_TTF
+int setTTFCodePage(void);
+#endif
+
 #endif /*DOSBOX_SDLMAIN_H*/
