@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -426,6 +426,8 @@ static void INLINE dyn_get_modrm(void) {
 #define MOV_REG_BYTE_TO_HOST_REG_LOW_CANUSEWORD(host_reg, reg_index, high_byte) gen_mov_regbyte_to_reg_low_canuseword(host_reg,(DRC_PTR_SIZE_IM)(DRCD_REG_BYTE(reg_index,high_byte)) - (DRC_PTR_SIZE_IM)(&cpu_regs))
 #define MOV_REG_BYTE_FROM_HOST_REG_LOW(host_reg, reg_index, high_byte) gen_mov_regbyte_from_reg_low(host_reg,(DRC_PTR_SIZE_IM)(DRCD_REG_BYTE(reg_index,high_byte)) - (DRC_PTR_SIZE_IM)(&cpu_regs))
 
+#define MOV_FLAGS_TO_HOST_REG(host_reg) gen_mov_regval32_to_reg(host_reg,(DRC_PTR_SIZE_IM)(DRCD_REG_FLAGS) - (DRC_PTR_SIZE_IM)(&cpu_regs))
+
 #else
 
 #define MOV_REG_VAL_TO_HOST_REG(host_reg, reg_index) gen_mov_word_to_reg(host_reg,DRCD_REG_VAL(reg_index),true)
@@ -443,6 +445,8 @@ static void INLINE dyn_get_modrm(void) {
 #define MOV_REG_BYTE_TO_HOST_REG_LOW_CANUSEWORD(host_reg, reg_index, high_byte) gen_mov_byte_to_reg_low_canuseword(host_reg,DRCD_REG_BYTE(reg_index,high_byte))
 #define MOV_REG_BYTE_FROM_HOST_REG_LOW(host_reg, reg_index, high_byte) gen_mov_byte_from_reg_low(host_reg,DRCD_REG_BYTE(reg_index,high_byte))
 
+#define MOV_FLAGS_TO_HOST_REG(host_reg) gen_mov_word_to_reg(host_reg,DRCD_REG_FLAGS,true)
+
 #endif
 
 
@@ -610,7 +614,7 @@ template <typename T> static DRC_PTR_SIZE_IM INLINE gen_call_function_mm(const T
 
 
 
-enum save_info_type {db_exception, cycle_check, string_break};
+enum save_info_type {db_exception, cycle_check, string_break, trap};
 
 
 // function that is called on exceptions
@@ -670,6 +674,13 @@ static void dyn_fill_blocks(void) {
 				gen_add_direct_word(&reg_eip,save_info_dynrec[sct].eip_change,decode.big_op);
 				dyn_return(BR_Cycles);
 				break;
+			case trap:
+				// trapflag is set, switch to trap-aware decoder
+				decode.cycles=save_info_dynrec[sct].cycles;
+				dyn_reduce_cycles();
+				gen_add_direct_word(&reg_eip,save_info_dynrec[sct].eip_change,decode.big_op);
+				dyn_return(BR_Trap);
+				break;
 		}
 	}
 	used_save_info_dynrec=0;
@@ -698,6 +709,19 @@ static void dyn_check_exception(HostReg reg) {
 }
 
 
+// check and branch if the trap flag is turned on
+static void dyn_check_trapflag(void) {
+	MOV_FLAGS_TO_HOST_REG(FC_OP1);
+	gen_and_imm(FC_OP1, FLAG_TF);
+	save_info_dynrec[used_save_info_dynrec].branch_pos=gen_create_branch_long_nonzero(FC_OP1,true);
+	save_info_dynrec[used_save_info_dynrec].cycles=decode.cycles;
+	save_info_dynrec[used_save_info_dynrec].eip_change=decode.code-decode.code_start;
+	if (!cpu.code.big) save_info_dynrec[used_save_info_dynrec].eip_change&=0xffff;
+	save_info_dynrec[used_save_info_dynrec].type=trap;
+	used_save_info_dynrec++;
+}
+
+
 
 bool DRC_CALL_CONV mem_readb_checked_drc(PhysPt address) DRC_FC;
 bool DRC_CALL_CONV mem_readb_checked_drc(PhysPt address) {
@@ -1151,28 +1175,6 @@ skip_extend_word:
 	}
 }
 
-
-
-// add code that checks if port access is allowed
-// the port is given in a register
-static void dyn_add_iocheck(HostReg reg_port,Bitu access_size) {
-	if (cpu.pmode) {
-		gen_call_function_RI(CPU_IO_Exception,reg_port,access_size);
-		dyn_check_exception(FC_RETOP);
-	}
-}
-
-// add code that checks if port access is allowed
-// the port is a constant
-static void dyn_add_iocheck_var(uint8_t accessed_port,Bitu access_size) {
-	if (cpu.pmode) {
-		gen_call_function_II(CPU_IO_Exception,accessed_port,access_size);
-		dyn_check_exception(FC_RETOP);
-	}
-}
-
-
-
 // save back the address register
 static void gen_protect_addr_reg(void) {
 #ifdef DRC_PROTECT_ADDR_REG
