@@ -26,6 +26,16 @@
 
 bool CPU_RDMSR();
 bool CPU_WRMSR();
+bool CPU_SYSENTER();
+bool CPU_SYSEXIT();
+
+/* wrap original variable name in inline function prior to #define, use inline function name in redefine of do_seg_limits */
+static inline bool _seg_limit_check(void) {
+	return do_seg_limits;
+}
+
+/* Do not emulate segment limit exceptions on 80186, DO emulate on 286. */
+#define do_seg_limits (_seg_limit_check() && CPU_ArchitectureType >= CPU_ARCHTYPE_286)
 
 #define PRE_EXCEPTION { }
 
@@ -88,7 +98,12 @@ extern Bitu cycle_count;
 #define TEST_PREFIX_REP		(core.prefixes & PREFIX_REP)
 
 #define DO_PREFIX_SEG(_SEG)					\
-    if (GETFLAG(IF) && CPU_Cycles <= 0 && !mustCompleteInstruction) goto prefix_out; \
+    if (GETFLAG(IF) && CPU_Cycles <= 0 && !mustCompleteInstruction) \
+    {   \
+        uint8_t next = LoadMb(core.cseip + 1);  \
+        if (next == 0xf2 || next == 0xf3)   \
+            goto prefix_out;    \
+    }   \
 	BaseDS=SegBase(_SEG);					\
 	BaseSS=SegBase(_SEG);					\
 	core.base_val_ds=_SEG;					\
