@@ -29,6 +29,10 @@ using namespace std;
 
 #include <algorithm>
 
+/* Do not emulate segment limit exceptions on 8086. This turns the if() statements into if (0)
+ * and the C++ compiler optimizer should then completely omit the code for the 8086 cputype. */
+#define do_seg_limits (0)
+
 /* 8086: Apparently exceptions like DIVIDE BY ZERO are given a return address of the next instruction */
 #define PRE_EXCEPTION SAVEIP
 
@@ -129,7 +133,12 @@ Bits CPU_Core_Prefetch_Trap_Run(void);
 #define TEST_PREFIX_REP		(core.prefixes & PREFIX_REP)
 
 #define DO_PREFIX_SEG(_SEG)					\
-    if (GETFLAG(IF) && CPU_Cycles <= 0) goto prefix_out; \
+    if (GETFLAG(IF) && CPU_Cycles <= 0) \
+    {   \
+        uint8_t next = LoadMb(core.cseip + 1);  \
+        if (next == 0xf2 || next == 0xf3)   \
+            goto prefix_out;    \
+    }   \
 	BaseDS=SegBase(_SEG);					\
 	BaseSS=SegBase(_SEG);					\
 	core.base_val_ds=_SEG;					\
@@ -262,15 +271,17 @@ restart_opcode:
 		invalidate_pq=false;
 		if (core.opcode_index&OPCODE_0F) invalidate_pq=true;
 		else switch (next_opcode) {
+            case 0x60:  case 0x61:  case 0x62:  case 0x63:
+            case 0x64:  case 0x65:  case 0x66:  case 0x67:
+            case 0x68:  case 0x69:  case 0x6a:  case 0x6b:
+            case 0x6c:  case 0x6d:  case 0x6e:  case 0x6f:
 			case 0x70:	case 0x71:	case 0x72:	case 0x73:
 			case 0x74:	case 0x75:	case 0x76:	case 0x77:
 			case 0x78:	case 0x79:	case 0x7a:	case 0x7b:
 			case 0x7c:	case 0x7d:	case 0x7e:	case 0x7f:	// jcc
 			case 0x9a:	// call
-			case 0xc2:	case 0xc3:	// retn
-			case 0xc8:	// enter
-			case 0xc9:	// leave
-			case 0xca:	case 0xcb:	// retf
+            case 0xc0:  case 0xc1:  case 0xc2:  case 0xc3:  // retn
+            case 0xc8:  case 0xc9:  case 0xca:  case 0xcb:  // retf
 			case 0xcc:	// int3
 			case 0xcd:	// int
 			case 0xce:	// into
@@ -314,6 +325,7 @@ restart_opcode:
 		}
 		SAVEIP;
 	}
+skip_saveip:
 
 #ifdef PREFETCH_DEBUG
     if (PIC_FullIndex() > pq_next_dbg) {
