@@ -171,6 +171,8 @@ static int addr32bit=0;
         w - word
 +       x - sign extended byte
 	F - use floating regs in mod/rm
+	M - use MMX regs in mod/rm
+	Q - qword
 	1-8 - group number, esc value, etc
 */
 
@@ -284,21 +286,27 @@ static char const *second[] = {
   0,                  0,                 0,                0,
 /* 3 */
   0,                  "rdtsc",           0,                0,
-  0,                  0,                 0,                0,
+  "sysenter",         "sysexit",         0,                0,
   0,                  0,                 0,                0,
   0,                  0,                 0,                0,
 /* 4 */
-  0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0,
+  "cmovo %Gv,%Ev",    "cmovno %Gv,%Ev",  "cmovc %Gv,%Ev",  "cmovnc %Gv,%Ev",
+  "cmovz %Gv,%Ev",    "cmovnz %Gv,%Ev",  "cmovbe %Gv,%Ev", "cmovnbe %Gv,%Ev",
+  "cmovs %Gv,%Ev",    "cmovns %Gv,%Ev",  "cmovp %Gv,%Ev",  "cmovnp %Gv,%Ev",
+  "cmovl %Gv,%Ev",    "cmovge %Gv,%Ev",  "cmovle %Gv,%Ev", "cmovg %Gv,%Ev",
 /* 5 */
   0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0,
 /* 6 */
-  0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0,
+  "punpcklbw %GM,%EM","punpcklwd %GM,%EM","punpckldq %GM,%EM","packsswb %GM,%EM",
+  "pcmpgtb %GM,%EM",  "pcmpgtw %GM,%EM", "pcmpgtd %GM,%EM","packuswb %GM,%EM",
+  "punpckhbw %GM,%EM","punpckhwd %GM,%EM","punpckhdq %GM,%EM","packssdw %GM,%EM",
+  0,                  0,                 "movd %GM,%Ed",   "movq %GM,%EM",
 /* 7 */
-  0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0,
+  0,                  "%g;",             "%g:",            "%g9",
+  "pcmpeqb %GM,%EM",  "pcmpeqw %GM,%EM", "pcmpeqd %GM,%EM","emms",
+  0,                  0,                 0,                0,
+  0,                  0,                 "movd %Ed,%GM",   "movq %EM,%GM",
 /* 8 */
   "jo %Jv",           "jno %Jv",         "jb %Jv",         "jnb %Jv",
   "jz %Jv",           "jnz %Jv",         "jbe %Jv",        "ja %Jv",
@@ -321,18 +329,24 @@ static char const *second[] = {
   "bsf %Gv,%Ev",      "bsr %Gv,%Ev",     "movsx %Gv,%Eb",  "movsx %Gv,%Ew",
 /* c */
   "xadd %Eb,%Gb",     "xadd %Ev,%Gv",    0,                0,
-  0,                  0,                 0,                0,
+  0,                  0,                 0,                "%g8",
   "bswap eax",        "bswap ecx",       "bswap edx",      "bswap ebx",
   "bswap esp",        "bswap ebp",       "bswap esi",      "bswap edi",
 /* d */
-  0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0,
+  0,                  "psrlw %GM,%EM",   "psrld %GM,%EM",  "psrlq %GM,%EM",
+  "paddq %GM,%EM",    "pmullw %GM,%EM",  0,                0,
+  "psubusb %GM,%EM",  "psubusw %GM,%EM", 0,                "pand %GM,%EM",
+  "paddusb %GM,%EM",  "paddusw %GM,%EM", 0,                "pandn %GM,%EM",
 /* e */
-  0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0,
+  0,                  "psraw %GM,%EM",   "psrad %GM,%EM",  0,
+  0,                  "pmulhw %GM,%EM",  0,                0,
+  "psubsb %GM,%EM",   "psubsw %GM,%EM",  0,                "por %GM,%EM",
+  "paddsb %GM,%EM",   "paddsw %GM,%EM",  0,                "pxor %GM,%EM",
 /* f */
-  0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0,
+  0,                  "psllw %GM,%EM",   "pslld %GM,%EM",  "psllq %GM,%EM",
+  0,                  "pmaddwd %GM,%EM", 0,                0,
+  "psubb %GM,%EM",    "psubw %GM,%EM",   "psubd %GM,%EM",  0,
+  "paddb %GM,%EM",    "paddw %GM,%EM",   "paddd %GM,%EM",  0
 };
 
 static char const *groups[][8] = {   /* group 0 is group 3 for %Ev set */
@@ -349,7 +363,7 @@ static char const *groups[][8] = {   /* group 0 is group 3 for %Ev set */
   { "inc %Eb",        "dec %Eb",         0,                0,
     0,                0,                 0,                "callback %Iw"  },
 /* 4 */
-  { "inc %Ev",        "dec %Ev",         "call %Kn%Ev",  "call %Kf%Ep",
+  { "inc %Ev",        "dec %Ev",         "call %Kn%Ev",    "call %Kf%Ep",
     "jmp %Kn%Ev",     "jmp %Kf%Ep",      "push %Ev",       0               },
 /* 5 */
   { "sldt %Ew",       "str %Ew",         "lldt %Ew",       "ltr %Ew",
@@ -359,7 +373,20 @@ static char const *groups[][8] = {   /* group 0 is group 3 for %Ev set */
     "smsw %Ew",       0,                 "lmsw %Ew",       "invlpg"        },
 /* 7 */
   { 0,                0,                 0,                0,
-    "bt",             "bts",             "btr",            "btc"           }
+    "bt",             "bts",             "btr",            "btc"           },
+/* 8 */
+  { 0,                "cmpxchg8b %EQ",   0,                0,
+    0,                0,                 0,                0               },
+/* 9 */
+  { 0,                0,                 "psrlq %EM,%Ib",  0,
+    0,                0,                 "psllq %EM,%Ib",  0               },
+/* : (NTS: this is '0'+10 in ASCII) */
+  { 0,                0,                 "psrld %EM,%Ib",  0,
+    "psrad %EM,%Ib",  0,                 "pslld %EM,%Ib",  0               },
+/* ; (NTS: this is '0'+11 in ASCII) */
+  { 0,                0,                 "psrlw %EM,%Ib",  0,
+    "psraw %EM,%Ib",  0,                 "psllw %EM,%Ib",  0               }
+
 };
 
 /* zero here means invalid.  If first entry starts with '*', use st(i) */
@@ -368,7 +395,7 @@ static char const *f0[]     = { 0, 0, 0, 0, 0, 0, 0, 0};
 static char const *fop_8[]  = { "*fld st,%GF" };
 static char const *fop_9[]  = { "*fxch st,%GF" };
 static char const *fop_10[] = { "fnop", 0, 0, 0, 0, 0, 0, 0 };
-static char const *fop_11[]  = { "*fst st,%GF" };
+static char const *fop_11[] = { "*fstp st,%GF" };
 static char const *fop_12[] = { "fchs", "fabs", 0, 0, "ftst", "fxam", 0, 0 };
 static char const *fop_13[] = { "fld1", "fldl2t", "fldl2e", "fldpi",
                    "fldlg2", "fldln2", "fldz", 0 };
@@ -377,7 +404,7 @@ static char const *fop_14[] = { "f2xm1", "fyl2x", "fptan", "fpatan",
 static char const *fop_15[] = { "fprem", "fyl2xp1", "fsqrt", "fsincos",
                    "frndint", "fscale", "fsin", "fcos" };
 static char const *fop_21[] = { 0, "fucompp", 0, 0, 0, 0, 0, 0 };
-static char const *fop_28[] = { "[fneni]", "[fndis]", "fclex", "finit", "[fnsetpm]", "[frstpm]", 0, 0 };
+static char const *fop_28[] = { "[fneni]", "[fndisi]", "fclex", "finit", "[fnsetpm]", "[frstpm]", 0, 0 };
 static char const* fop_29[] = { "*fucomi %GF" };
 static char const* fop_30[] = { "*fcomi %GF" };
 static char const *fop_32[] = { "*fadd %GF,st" };
@@ -403,6 +430,9 @@ static char const *fop_53[] = { "*fsubp %GF,st" };
 static char const *fop_54[] = { "*fdivrp %GF,st" };
 static char const *fop_55[] = { "*fdivp %GF,st" };
 static char const *fop_56[] = { "*ffreep %GF" };
+static char const *fop_57[] = { "*fxch %GF" };
+static char const *fop_58[] = { "*fstp %GF" };
+static char const *fop_59[] = { "*fstp %GF" };
 static char const *fop_60[] = { "fstsw ax", 0, 0, 0, 0, 0, 0, 0 };
 static char const* fop_61[] = { "*fucomip %GF" };
 static char const* fop_62[] = { "*fcomip %GF" };
@@ -415,7 +445,7 @@ static char const **fspecial[] = { /* 0=use st(i), 1=undefined 0 in fop_* means
   fop_32, fop_33, fop_34, fop_35, fop_36, fop_37, fop_38, fop_39,
   fop_40, fop_41, fop_42, fop_43, fop_44, fop_45, f0, f0,
   fop_48, fop_49, fop_50, fop_51, fop_52, fop_53, fop_54, fop_55,
-  fop_56, f0, f0, f0, fop_60, fop_61, fop_62, f0,
+  fop_56, fop_57, fop_58, fop_59, fop_60, fop_61, fop_62, f0,
 };
 
 static const char *floatops[] = { /* assumed " %EF" at end of each.  mod != 3 only */
@@ -423,16 +453,16 @@ static const char *floatops[] = { /* assumed " %EF" at end of each.  mod != 3 on
        "fsub", "fsubr", "fdiv", "fdivr",
 /*08*/ "fld", 0, "fst", "fstp",
        "fldenv", "fldcw", "fstenv", "fstcw",
-/*16*/ "fiadd", "fimul", "ficomw", "ficompw",
+/*16*/ "fiadd", "fimul", "ficom", "ficomp",
        "fisub", "fisubr", "fidiv", "fidivr",
 /*24*/ "fild", "fisttp", "fist", "fistp",
-       "frstor", "fldt", 0, "fstpt",
+       0, "fldt", 0, "fstpt",
 /*32*/ "faddq", "fmulq", "fcomq", "fcompq",
        "fsubq", "fsubrq", "fdivq", "fdivrq",
 /*40*/ "fldq", "fisttpq", "fstq", "fstpq",
-       0, 0, "fsave", "fstsw",
+       "frstor", 0, "fsave", "fstsw",
 /*48*/ "fiaddw", "fimulw", "ficomw", "ficompw",
-       "fisubw", "fisubrw", "fidivw", "fidivr",
+       "fisubw", "fisubrw", "fidivw", "fidivrw",
 /*56*/ "fildw", "fisttpw", "fistw", "fistpw",
        "fbldt", "fildq", "fbstpt", "fistpq"
 };
@@ -672,6 +702,10 @@ static void reg_name(int regnum, char size)
     uprintf("st(%d)", regnum);
     return;
   }
+  if (size == 'M') { /* MMX register */
+    uprintf("mm%d", regnum);
+    return;
+  }
   if ((((size == 'c') || (size == 'v')) && (opsize == 32)) || (size == 'd'))
     uputchar('e');
   if ((size=='q' || size == 'b' || size=='c') && !wordop) {
@@ -746,7 +780,10 @@ static void do_modrm(char subtype)
     return;
   }
   if (must_do_size) {
-    if (wordop) {
+    if (subtype == 'Q') {
+	  ua_str("qword ");
+    }
+    else if (wordop) {
       if (addrsize==32 || opsize==32) {       /* then must specify size */
 		ua_str("dword ");
       } else {
@@ -811,16 +848,11 @@ static void floating_point(int e1)
 {
   int esc = e1*8 + REG(modrm());
 
-  if ((MOD(modrm()) == 3)&&fspecial[esc]) {
-    if (fspecial[esc][0]) {
-      if (fspecial[esc][0][0] == '*') {
-        ua_str(fspecial[esc][0]+1);
-      } else {
-        ua_str(fspecial[esc][RM(modrm())]);
-      }
+  if ((MOD(modrm()) == 3) && fspecial[esc]) {
+    if (fspecial[esc][0] && fspecial[esc][0][0] == '*') {
+      ua_str(fspecial[esc][0]+1);
     } else {
-      ua_str(floatops[esc]);
-      ua_str(" %EF");
+      ua_str(fspecial[esc][RM(modrm())]);
     }
   } else {
     ua_str(floatops[esc]);
