@@ -96,6 +96,7 @@ static void RestoreRegisters(void) {
 	reg_sp+=18;
 }
 
+extern bool force_sfn;
 extern uint8_t ZDRIVE_NUM;
 extern void GFX_SetTitle(int32_t cycles, int frameskip, Bits timing, bool paused), menu_update_autocycle(void);
 void DOS_UpdatePSPName(void) {
@@ -356,11 +357,14 @@ bool DOS_Execute(const char* name, PhysPt block_pt, uint8_t flags) {
 	if (flags!=OVERLAY) {
 		/* Create an environment block */
 		envseg=block.exec.envseg;
+		force_sfn = true;
 		if (!MakeEnv(name,&envseg)) {
 			DOS_CloseFile(fhandle);
 			delete [] loadbuf;
+			force_sfn = false;
 			return false;
 		}
+		force_sfn = false;
 		uint16_t minsize;
 		/* Get Memory */		
 		DOS_AllocateMemory(&pspseg,&maxfree);
