@@ -379,8 +379,8 @@ static bool DOS_MultiplexFunctions(void) {
 		char psp_name[9];
 		DOS_MCB psp_mcb(dos.psp()-1);
 		psp_mcb.GetFileName(psp_name);
-		// Report Windows version 4.0 (95) to NESTICLE x.xx so that it uses LFN when available
-		if (uselfn && (!strcmp(psp_name, "NESTICLE") || (reg_sp == 0x220A && mem_readw(SegPhys(ss)+reg_sp)/0x100 == 0x1F))) {
+		// Report Windows version 4.0 (95) to PEDIT and NESTICLE x.xx so that they use LFN when available
+		if (uselfn && (!strcmp(psp_name, "PEDIT") || !strcmp(psp_name, "PEDITLGT") || ((!strcmp(psp_name, "EDIT") || reg_sp/0x100 == 0xF) && mem_readw(SegPhys(ss)+reg_sp) == 0x4A) || !strcmp(psp_name, "NESTICLE") || (reg_sp == 0x220A && mem_readw(SegPhys(ss)+reg_sp)/0x100 == 0x1F))) {
 			reg_ax = 0;
 			reg_bx = 0x400;
 			reg_cx = 2;
@@ -397,6 +397,13 @@ static bool DOS_MultiplexFunctions(void) {
 		return true;
 	case 0x1700:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
+        {// Norton Utilities 8.0 installer checks this before continue
+            char psp_name[9];
+            DOS_MCB psp_mcb(dos.psp()-1);
+            psp_mcb.GetFileName(psp_name);
+	    // NTS: DEFRAG.EXE for MS-DOS 6.22 assumes Windows is running if this call responds affirmatively, because it would mean WINOLDAP is resident.
+            if (((!strcmp(psp_name, "INSTALL") || !strcmp(psp_name, "INSTALLD")) && reg_sp >= 0xD000 && mem_readw(SegPhys(ss)+reg_sp)/0x100 == 0x1E) || !strcmp(psp_name, "DISKEDIT") || !strcmp(psp_name, "NDD") || !strcmp(psp_name, "NDIAGS") || !strcmp(psp_name, "UNERASE") || !strcmp(psp_name, "UNFORMAT") || !strcmp(psp_name,"DEFRAG")) return false;
+        }
 		reg_al = 1;
 		reg_ah = 1;
 		return true;
