@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,15 +25,15 @@
 #include "paging.h"
 #include "cross.h"
 #include "mem.h"
-#include "fpu.h"
 #include "cpu.h"
+#include "fpu.h"
 #include "../cpu/lazyflags.h"
 
 FPU_rec fpu;
 
-void FPU_FLDCW(PhysPt addr){
-	uint16_t temp = mem_readw(addr);
-	FPU_SetCW(temp);
+void FPU_FLDCW(PhysPt addr)
+{
+	fpu.cw = mem_readw(addr);
 }
 
 uint16_t FPU_GetTag(void){
@@ -382,7 +382,13 @@ void FPU_ESC3_EA(Bitu rm,PhysPt addr) {
 		}
 		break;
 	case 0x01:	/* FISTTP */
-		LOG(LOG_FPU,LOG_WARN)("ESC 3 EA:Unhandled group %d subfunction %d",(int)group,(int)sub);
+        if(CPU_ArchitectureType == CPU_ARCHTYPE_EXPERIMENTAL)
+        {
+            FPU_FSTT_I32(addr);
+            FPU_FPOP();
+        }
+        else
+            LOG(LOG_FPU, LOG_WARN)("ESC 3 EA:Unhandled group %d subfunction %d", (int)group, (int)sub);
 		break;
 	case 0x02:	/* FIST */
 		FPU_FST_I32(addr);
@@ -434,8 +440,16 @@ void FPU_ESC3_Normal(Bitu rm) {
 	case 0x04:
 		switch (sub) {
 		case 0x00:				//FNENI
+			if (CPU_ArchitectureType==CPU_ARCHTYPE_8086)
+				fpu.cw.M = false;
+			else
+				LOG(LOG_FPU,LOG_ERROR)("8087 only fpu code used esc 3: group 4: subfuntion :%d",(int)sub);
+			break;
 		case 0x01:				//FNDIS
-			LOG(LOG_FPU,LOG_ERROR)("8087 only fpu code used esc 3: group 4: subfuntion :%d",(int)sub);
+			if (CPU_ArchitectureType==CPU_ARCHTYPE_8086)
+				fpu.cw.M = true;
+			else
+				LOG(LOG_FPU,LOG_ERROR)("8087 only fpu code used esc 3: group 4: subfuntion :%d",(int)sub);
 			break;
 		case 0x02:				//FNCLEX FCLEX
 			FPU_FCLEX();
@@ -527,7 +541,13 @@ void FPU_ESC5_EA(Bitu rm,PhysPt addr) {
 		}
 		break;
 	case 0x01:  /* FISTTP longint*/
-		LOG(LOG_FPU,LOG_WARN)("ESC 5 EA:Unhandled group %d subfunction %d",(int)group,(int)sub);
+        if(CPU_ArchitectureType == CPU_ARCHTYPE_EXPERIMENTAL)
+        {
+            FPU_FSTT_I64(addr);
+            FPU_FPOP();
+        }
+        else
+            LOG(LOG_FPU, LOG_WARN)("ESC 5 EA:Unhandled group %d subfunction %d", (int)group, (int)sub);
 		break;
 	case 0x02:   /* FST double real*/
 		FPU_FST_F64(addr);
@@ -649,8 +669,14 @@ void FPU_ESC7_EA(Bitu rm,PhysPt addr) {
 			}
 		}
 		break;
-	case 0x01:
-		LOG(LOG_FPU,LOG_WARN)("ESC 7 EA:Unhandled group %d subfunction %d",(int)group,(int)sub);
+	case 0x01:  /* FISTTP int16_t */
+        if(CPU_ArchitectureType == CPU_ARCHTYPE_EXPERIMENTAL)
+        {
+            FPU_FSTT_I16(addr);
+            FPU_FPOP();
+        }
+        else
+            LOG(LOG_FPU, LOG_WARN)("ESC 7 EA:Unhandled group %d subfunction %d", (int)group, (int)sub);
 		break;
 	case 0x02:   /* FIST int16_t */
 		FPU_FST_I16(addr);
