@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,11 @@
 #include "cpu/lazyflags.h"
 
 #ifdef __GNUC__
-# if defined(__MINGW32__) || defined(MACOSX)
+# if defined(__MINGW32__) || (defined(MACOSX) && !defined(__arm64__))
 #  include "fpu_control_x86.h"
+# elif defined(ANDROID) || defined(__ANDROID__) || (defined(MACOSX) && defined(__arm64__)) || defined(EMSCRIPTEN)
+/* ? */
+#  define _FPU_SETCW(x) /* dummy */
 # else
 #  include <fpu_control.h>
 # endif
@@ -35,7 +38,7 @@ static inline void FPU_SyncCW(void) {
 #endif
 
 static void FPU_FINIT(void) {
-	FPU_SetCW(0x37F);
+	fpu.cw.init();
     FPU_SyncCW();
     fpu.sw = 0;
 	TOP=FPU_GET_TOP();
@@ -81,19 +84,19 @@ static void FPU_FPOP(void){
 }
 
 static long double FROUND(long double in){
-	switch(fpu.round){
-	case ROUND_Nearest:	
+	switch(fpu.cw.RC){
+	case FPUControlWord::RoundMode::Nearest:
 		if (in-floorl(in)>0.5) return (floorl(in)+1);
 		else if (in-floorl(in)<0.5) return (floorl(in));
 		else return (((static_cast<int64_t>(floorl(in)))&1)!=0)?(floorl(in)+1):(floorl(in));
 		break;
-	case ROUND_Down:
+	case FPUControlWord::RoundMode::Down:
 		return (floorl(in));
 		break;
-	case ROUND_Up:
+	case FPUControlWord::RoundMode::Up:
 		return (ceill(in));
 		break;
-	case ROUND_Chop:
+	case FPUControlWord::RoundMode::Chop:
 		return in; //the cast afterwards will do it right maybe cast here
 		break;
 	default:
@@ -228,6 +231,24 @@ static void FPU_FST_I64(PhysPt addr) {
 	mem_writeq(addr,(uint64_t)static_cast<int64_t>(FROUND(fpu.regs_80[TOP].v)));
 }
 
+// WARNING: UNTESTED. Original contributed code only focused on the x86 FPU case.
+static void FPU_FSTT_I16(PhysPt addr) {
+	mem_writew(addr,(uint16_t)static_cast<int16_t>(fpu.regs_80[TOP].v));
+	FPU_FPOP();
+}
+
+// WARNING: UNTESTED. Original contributed code only focused on the x86 FPU case.
+static void FPU_FSTT_I32(PhysPt addr) {
+	mem_writed(addr,(uint32_t)static_cast<int32_t>(fpu.regs_80[TOP].v));
+	FPU_FPOP();
+}
+
+// WARNING: UNTESTED. Original contributed code only focused on the x86 FPU case.
+static void FPU_FSTT_I64(PhysPt addr) {
+	mem_writeq(addr,(uint64_t)static_cast<int64_t>(fpu.regs_80[TOP].v));
+	FPU_FPOP();
+}
+
 static void FPU_FBST(PhysPt addr) {
 	FPU_Reg_80 val = fpu.regs_80[TOP];
 	bool sign = false;
@@ -545,7 +566,7 @@ static void FPU_FLDENV(PhysPt addr){
 		tag    = static_cast<uint16_t>(tagbig);
 	}
 	FPU_SetTag(tag);
-	FPU_SetCW(cw);
+	fpu.cw = cw;
     FPU_SyncCW();
 	TOP = FPU_GET_TOP();
 }
