@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,10 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-
+#include "cpu.h"
+#include "fpu.h"
+#include "mem.h"
+#include "regs.h"
 
 // #define WEAK_EXCEPTIONS
 
@@ -73,11 +76,11 @@
 
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_STORE(op,szI,szA)				\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
 		__asm	mov		eax, TOP			\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked           \
 		__asm	shl		eax, 4				\
 		__asm	fld		TBYTE PTR fpu.p_regs[eax].m1	\
 		__asm	op		szI PTR fpu.p_regs[128].m1		\
@@ -85,10 +88,10 @@
 		}
 #else
 #define FPUD_STORE(op,szI,szA)				\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked           \
 		__asm	mov		eax, TOP			\
 		__asm	shl		eax, 4				\
 		__asm	mov		ebx, 8				\
@@ -210,12 +213,12 @@
 // handles fadd,fmul,fsub,fsubr
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_ARITH1(op)						\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
 		__asm	mov		eax, op1			\
 		__asm	shl		eax, 4				\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked           \
 		__asm	mov		ebx, op2			\
 		__asm	shl		ebx, 4				\
 		__asm	fld		TBYTE PTR fpu.p_regs[eax].m1	\
@@ -226,10 +229,10 @@
 		}
 #else
 #define FPUD_ARITH1(op)						\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked			\
 		__asm	mov		eax, op1			\
 		__asm	shl		eax, 4				\
 		__asm	mov		ebx, op2			\
@@ -248,11 +251,11 @@
 // handles fadd,fmul,fsub,fsubr
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_ARITH1_EA(op)					\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
 		__asm	mov		eax, op1			\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked    		\
 		__asm	shl		eax, 4				\
 		__asm	fld		TBYTE PTR fpu.p_regs[eax].m1	\
 		__asm	fxch	\
@@ -262,10 +265,10 @@
 		}
 #else
 #define FPUD_ARITH1_EA(op)					\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked    		\
 		__asm	mov		eax, op1			\
 		__asm	shl		eax, 4				\
 		__asm	fld		TBYTE PTR fpu.p_regs[eax].m1	\
@@ -282,11 +285,11 @@
 // handles fsqrt,frndint
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_ARITH2(op)						\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
 		__asm	mov		eax, TOP			\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked			\
 		__asm	shl		eax, 4				\
 		__asm	fld		TBYTE PTR fpu.p_regs[eax].m1	\
 		__asm	op							\
@@ -295,10 +298,10 @@
 		}
 #else
 #define FPUD_ARITH2(op)						\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked			\
 		__asm	mov		eax, TOP			\
 		__asm	shl		eax, 4				\
 		__asm	fld		TBYTE PTR fpu.p_regs[eax].m1	\
@@ -314,10 +317,10 @@
 // handles fdiv,fdivr
 // (This is identical to FPUD_ARITH1 but without a WEAK_EXCEPTIONS variant)
 #define FPUD_ARITH3(op)						\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked			\
 		__asm	mov		eax, op1			\
 		__asm	shl		eax, 4				\
 		__asm	mov		ebx, op2			\
@@ -335,11 +338,11 @@
 // handles fdiv,fdivr
 // (This is identical to FPUD_ARITH1_EA but without a WEAK_EXCEPTIONS variant)
 #define FPUD_ARITH3_EA(op)					\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm {								\
 		__asm	fnstcw	save_cw				\
 		__asm	mov		eax, op1			\
-		__asm	fldcw	fpu.cw_mask_all		\
+		__asm	fldcw	cw_masked			\
 		__asm	shl		eax, 4				\
 		__asm	fld		TBYTE PTR fpu.p_regs[eax].m1	\
 		__asm	fxch	\
@@ -553,7 +556,7 @@
 
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_STORE(op,szI,szA)				\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm__ volatile (					\
 			"fnstcw		%0				\n"	\
 			"fldcw		%3				\n"	\
@@ -561,11 +564,11 @@
 			#op #szA "	%1				\n"	\
 			"fldcw		%0				"	\
 			:	"=m" (save_cw), "=m" (fpu.p_regs[8])	\
-			:	"m" (fpu.p_regs[TOP]), "m" (fpu.cw_mask_all)		\
+			:	"m" (fpu.p_regs[TOP]), "m" (cw_masked)		\
 		);
 #else
 #define FPUD_STORE(op,szI,szA)				\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm__ volatile (					\
 			"fnstcw		%1				\n"	\
 			"fldcw		%4				\n"	\
@@ -575,7 +578,7 @@
 			"fnstsw		%0				\n"	\
 			"fldcw		%1				"	\
 			:	"=&am" (new_sw), "=m" (save_cw), "=m" (fpu.p_regs[8])	\
-			:	"m" (fpu.p_regs[TOP]), "m" (fpu.cw_mask_all)			\
+			:	"m" (fpu.p_regs[TOP]), "m" (cw_masked)			\
 		);									\
 		fpu.sw=(new_sw&exc_mask)|(fpu.sw&0x80ff);
 #endif
@@ -668,7 +671,7 @@
 // handles fadd,fmul,fsub,fsubr
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_ARITH1(op)						\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm__ volatile (					\
 			"fnstcw		%0				\n"	\
 			"fldcw		%3				\n"	\
@@ -678,11 +681,11 @@
 			"fstpt		%1				\n"	\
 			"fldcw		%0				"	\
 			:	"=m" (save_cw), "+m" (fpu.p_regs[op1])				\
-			:	"m" (fpu.p_regs[op2]), "m" (fpu.cw_mask_all)		\
+			:	"m" (fpu.p_regs[op2]), "m" (cw_masked)		\
 		);
 #else
 #define FPUD_ARITH1(op)						\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm__ volatile (					\
 			"fnstcw		%1				\n"	\
 			"fldcw		%4				\n"	\
@@ -694,7 +697,7 @@
 			"fstpt		%2				\n"	\
 			"fldcw		%1				"	\
 			:	"=&am" (new_sw), "=m" (save_cw), "+m" (fpu.p_regs[op1])	\
-			:	"m" (fpu.p_regs[op2]), "m" (fpu.cw_mask_all)		\
+			:	"m" (fpu.p_regs[op2]), "m" (cw_masked)		\
 		);									\
 		fpu.sw=(new_sw&exc_mask)|(fpu.sw&0x80ff);
 #endif
@@ -702,7 +705,7 @@
 // handles fadd,fmul,fsub,fsubr
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_ARITH1_EA(op)					\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm__ volatile (					\
 			"fnstcw		%0				\n"	\
 			"fldcw		%2				\n"	\
@@ -711,11 +714,11 @@
 			"fstpt		%1				\n"	\
 			"fldcw		%0				"	\
 			:	"=m" (save_cw), "+m" (fpu.p_regs[op1])		\
-			:	"m" (fpu.cw_mask_all)		\
+			:	"m" (cw_masked)		\
 		);
 #else
 #define FPUD_ARITH1_EA(op)					\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm__ volatile (					\
 			"fnstcw		%1				\n"	\
 			"fldcw		%3				\n"	\
@@ -726,7 +729,7 @@
 			"fstpt		%2				\n"	\
 			"fldcw		%1				"	\
 			:	"=&am" (new_sw), "=m" (save_cw), "+m" (fpu.p_regs[op1])	\
-			:	"m" (fpu.cw_mask_all)		\
+			:	"m" (cw_masked)		\
 		);									\
 		fpu.sw=(new_sw&exc_mask)|(fpu.sw&0x80ff);
 #endif
@@ -734,7 +737,7 @@
 // handles fsqrt,frndint
 #ifdef WEAK_EXCEPTIONS
 #define FPUD_ARITH2(op)						\
-		uint16_t save_cw;						\
+		uint16_t save_cw,cw_masked=fpu.cw.allMasked();						\
 		__asm__ volatile (					\
 			"fnstcw		%0				\n"	\
 			"fldcw		%2				\n"	\
@@ -743,11 +746,11 @@
 			"fstpt		%1				\n"	\
 			"fldcw		%0				"	\
 			:	"=m" (save_cw), "+m" (fpu.p_regs[TOP])		\
-			:	"m" (fpu.cw_mask_all)		\
+			:	"m" (cw_masked)		\
 		);
 #else
 #define FPUD_ARITH2(op)						\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm__ volatile (					\
 			"fnstcw		%1				\n"	\
 			"fldcw		%3				\n"	\
@@ -758,7 +761,7 @@
 			"fstpt		%2				\n"	\
 			"fldcw		%1				"	\
 			:	"=&am" (new_sw), "=m" (save_cw), "+m" (fpu.p_regs[TOP])	\
-			:	"m" (fpu.cw_mask_all)		\
+			:	"m" (cw_masked)		\
 		);										\
 		fpu.sw=(new_sw&exc_mask)|(fpu.sw&0x80ff);
 #endif
@@ -766,7 +769,7 @@
 // handles fdiv,fdivr
 // (This is identical to FPUD_ARITH1 but without a WEAK_EXCEPTIONS variant)
 #define FPUD_ARITH3(op)						\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm__ volatile (					\
 			"fnstcw		%1				\n"	\
 			"fldcw		%4				\n"	\
@@ -778,14 +781,14 @@
 			"fstpt		%2				\n"	\
 			"fldcw		%1				"	\
 			:	"=&am" (new_sw), "=m" (save_cw), "+m" (fpu.p_regs[op1])	\
-			:	"m" (fpu.p_regs[op2]), "m" (fpu.cw_mask_all)		\
+			:	"m" (fpu.p_regs[op2]), "m" (cw_masked)		\
 		);									\
 		fpu.sw=(new_sw&0xffbf)|(fpu.sw&0x80ff);
 
 // handles fdiv,fdivr
 // (This is identical to FPUD_ARITH1_EA but without a WEAK_EXCEPTIONS variant)
 #define FPUD_ARITH3_EA(op)					\
-		uint16_t new_sw,save_cw;				\
+		uint16_t new_sw,save_cw,cw_masked=fpu.cw.allMasked();				\
 		__asm__ volatile (					\
 			"fnstcw		%1				\n"	\
 			"fldcw		%3				\n"	\
@@ -796,7 +799,7 @@
 			"fstpt		%2				\n"	\
 			"fldcw		%1				"	\
 			:	"=&am" (new_sw), "=m" (save_cw), "+m" (fpu.p_regs[op1])	\
-			:	"m" (fpu.cw_mask_all)		\
+			:	"m" (cw_masked)		\
 		);									\
 		fpu.sw=(new_sw&0xffbf)|(fpu.sw&0x80ff);
 
@@ -934,7 +937,7 @@ const uint16_t exc_mask=0xffbf;
 #endif
 
 static void FPU_FINIT(void) {
-	FPU_SetCW(0x37F);
+	fpu.cw.init();
 	fpu.sw=0;
 	TOP=FPU_GET_TOP();
 	fpu.tags[0]=TAG_Empty;
@@ -1063,6 +1066,22 @@ static void FPU_FST_I64(PhysPt addr) {
 	mem_writed(addr+4,fpu.p_regs[8].m2);
 }
 
+static void FPU_FSTT_I16(PhysPt addr) {
+    FPUD_STORE(fisttp, WORD, s)
+    mem_writew(addr, (uint16_t)fpu.p_regs[8].m1);
+}
+
+static void FPU_FSTT_I32(PhysPt addr) {
+    FPUD_STORE(fisttp, DWORD, l)
+    mem_writed(addr, fpu.p_regs[8].m1);
+}
+
+static void FPU_FSTT_I64(PhysPt addr) {
+    FPUD_STORE(fisttp, QWORD, q)
+    mem_writed(addr, fpu.p_regs[8].m1);
+    mem_writed(addr + 4, fpu.p_regs[8].m2);
+}
+
 static void FPU_FBST(PhysPt addr) {
 	FPUD_STORE(fbstp,TBYTE,)
 	mem_writed(addr,fpu.p_regs[8].m1);
@@ -1284,7 +1303,7 @@ static void FPU_FLDENV(PhysPt addr){
 		tag    = static_cast<uint16_t>(tagbig);
 	}
 	FPU_SetTag(tag);
-	FPU_SetCW(cw);
+	fpu.cw = cw;
 	TOP=FPU_GET_TOP();
 }
 
