@@ -1227,13 +1227,14 @@ LPWSTR getWString(std::string str, wchar_t *def, wchar_t*& buffer) {
             if (IS_PC98_ARCH || IS_JEGA_ARCH || IS_JDOSV) cp = 932;
             else if (IS_PDOSV) cp = 936;
             else if (IS_KDOSV) cp = 949;
-            else if (IS_CDOSV) cp = 950;
+            else if (IS_TDOSV) cp = 950;
         }
     }
     uint16_t len=(uint16_t)str.size();
     if (cp>0) {
         if (cp==808) cp=866;
         else if (cp==872) cp=855;
+        else if (cp==951) cp=950;
         reqsize = MultiByteToWideChar(cp, 0, str.c_str(), len+1, NULL, 0);
         buffer = new wchar_t[reqsize];
         if (reqsize>0 && MultiByteToWideChar(cp, 0, str.c_str(), len+1, buffer, reqsize)==reqsize) ret = (LPWSTR)buffer;
