@@ -362,6 +362,7 @@ static void RENDER_Halt( void ) {
 
 extern Bitu PIC_Ticks;
 extern bool pause_on_vsync;
+extern bool checkmenuwidth;
 void PauseDOSBox(bool pressed);
 void AspectRatio_mapper_shortcut(bool pressed);
 
@@ -437,6 +438,12 @@ static Bitu MakeAspectTable(Bitu skip,Bitu height,double scaley,Bitu miny) {
     return linesadded;
 }
 
+std::string RENDER_GetScaler(void) {
+    Section_prop * section=static_cast<Section_prop *>(control->GetSection("render"));
+    Prop_multival* prop = section->Get_multival("scaler");
+    return prop->GetSection()->Get_string("type");
+}
+
 void RENDER_Reset( void ) {
     Bitu width=render.src.width;
     Bitu height=render.src.height;
@@ -445,6 +452,7 @@ void RENDER_Reset( void ) {
 
     double gfx_scalew;
     double gfx_scaleh;
+    const std::string scaler = RENDER_GetScaler();
 
     if (width == 0 || height == 0)
         return;
@@ -473,6 +481,14 @@ void RENDER_Reset( void ) {
     if( sdl.desktop.isperfect ) /* Handle scaling if no pixel-perfect mode */
         goto forcenormal;
 
+    if((!dblh || !dblw) && scaler != "none" && strncasecmp(scaler.c_str(), "normal", 6) && !render.scale.forced && sdl.desktop.want_type != SCREEN_TTF) {
+        std::string message = "This scaler may not work properly or have undesired effect:\n\n"+scaler+"\n\nDo you want to load the scaler anyway?\n\n(You may append 'forced' to the scaler setting to force load the scaler without this message)";
+        bool systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
+        if (systemmessagebox("Loading scaler", message.c_str(), "yesno","question", 1))
+            render.scale.forced = true;
+        else
+            SetVal("render", "scaler", "none");
+    }
     if ((dblh && dblw) || (render.scale.forced && dblh == dblw/*this branch works best with equal scaling in both directions*/)) {
         /* Initialize always working defaults */
         if (render.scale.size == 2)
@@ -863,6 +879,7 @@ void RENDER_SetSize(Bitu width,Bitu height,Bitu bpp,float fps,double scrn_ratio)
     render.src.fps=fps;
     render.src.ratio=ratio;
     render.src.scrn_ratio=scrn_ratio;
+    checkmenuwidth = true;
     RENDER_Reset( );
 }
 
@@ -1030,12 +1047,6 @@ void RENDER_OnSectionPropChange(Section *x) {
     RENDER_UpdateScalerMenu();
 }
 
-std::string RENDER_GetScaler(void) {
-    Section_prop * section=static_cast<Section_prop *>(control->GetSection("render"));
-    Prop_multival* prop = section->Get_multival("scaler");
-    return prop->GetSection()->Get_string("type");
-}
-
 extern const char *scaler_menu_opts[][2];
 
 void RENDER_UpdateScalerMenu(void) {
@@ -1310,17 +1321,14 @@ namespace
 class SerializeRender : public SerializeGlobalPOD
 {
 public:
-	SerializeRender() : SerializeGlobalPOD("Render")
-	{}
+	SerializeRender() : SerializeGlobalPOD("Render") {}
 
 private:
 	virtual void getBytes(std::ostream& stream)
 	{
-		SerializeGlobalPOD::getBytes(stream);
-
-
 		// - pure data
-		WRITE_POD( &render.src, render.src );
+        SerializeGlobalPOD::getBytes(stream);
+        WRITE_POD( &render.src, render.src );
 		WRITE_POD( &render.pal, render.pal );
 		WRITE_POD( &render.updating, render.updating );
 		WRITE_POD( &render.active, render.active );
@@ -1332,10 +1340,8 @@ private:
 
 	virtual void setBytes(std::istream& stream)
 	{
-		SerializeGlobalPOD::setBytes(stream);
-
-
 		// - pure data
+        SerializeGlobalPOD::setBytes(stream);
 		READ_POD( &render.src, render.src );
 		READ_POD( &render.pal, render.pal );
 		READ_POD( &render.updating, render.updating );
@@ -1343,24 +1349,27 @@ private:
 		READ_POD( &render.fullFrame, render.fullFrame );
 		READ_POD( &render.frameskip, render.frameskip );
 		READ_POD( &render.aspect, render.aspect );
-		scalerOperation_t op = render.scale.op;
+        scalerOperation_t op = render.scale.op;
 		Bitu size = render.scale.size;
 		bool hardware = render.scale.hardware;
-		READ_POD( &render.scale, render.scale );
-		render.scale.op = op;
+        uint8_t* cacheRead = render.scale.cacheRead;
+        uint8_t* outWrite = render.scale.outWrite;
+        Bitu cachePitch = render.scale.cachePitch;
+        Bitu outPitch = render.scale.outPitch;
+        READ_POD( &render.scale, render.scale );
+        render.scale.cacheRead = cacheRead;
+        render.scale.cachePitch = cachePitch;
+        render.scale.outWrite = outWrite;
+        render.scale.outPitch = outPitch;
+        render.scale.op = op;
 		render.scale.size = size;
 		render.scale.hardware = hardware;
-
-		//***************************************
-		//***************************************
-
+ 
 		// reset screen
-		//memset( &render.frameskip, 0, sizeof(render.frameskip) );
-
 		if (render.aspect==ASPECT_FALSE) {
 			render.scale.clearCache = true;
 			if( render.scale.outWrite ) { GFX_EndUpdate(NULL); }
-			RENDER_SetSize( render.src.width, render.src.height, render.src.bpp, render.src.fps, render.src.ratio );
+			RENDER_SetSize( render.src.width, render.src.height, render.src.bpp, render.src.fps, render.src.scrn_ratio );
 		} else
 			GFX_ResetScreen();
 	}
