@@ -1062,7 +1062,7 @@ private:
     };
 };
 
-static void MIXER_ProgramStart(Program * * make) {
+void MIXER_ProgramStart(Program * * make) {
     *make=new MIXER;
 }
 
@@ -1168,7 +1168,6 @@ void MIXER_Controls_Init() {
 }
 
 void MIXER_DOS_Boot(Section *) {
-    PROGRAMS_MakeFile("MIXER.COM",MIXER_ProgramStart,"/SYSTEM/");
 }
 
 void MIXER_Init() {
