@@ -298,6 +298,7 @@ static scancode_tbl scan_to_scanascii_pc98[0x80] = {
 #include <queue>
 std::queue <uint16_t>over_key_buffer;
 
+extern bool inshell;
 #if defined(USE_TTF)
 extern bool ttf_dosv;
 #endif
@@ -334,6 +335,33 @@ bool BIOS_AddKeyToBuffer(uint16_t code) {
     if (ttail>=end) {
         ttail=start;
     }
+    if(J3_IsJapanese()) {
+        if((code & 0xff) == 0xe0) {
+            if((code & 0xff00) >= 0x3b00 && (code & 0xff00) <= 0x5300) {
+                code &= 0xff00;
+            } else if((code & 0xff00) == 0x7300) {
+                code = 0x4b00;
+            } else if((code & 0xff00) == 0x7400) {
+                code = 0x4d00;
+            } else if((code & 0xff00) == 0x7500) {
+                code = 0x4f00;
+            } else if((code & 0xff00) == 0x7600) {
+                code = 0x4900;
+            } else if((code & 0xff00) == 0x7700) {
+                code = 0x4700;
+            } else if((code & 0xff00) == 0x8400) {
+                code = 0x5100;
+            } else if((code & 0xff00) == 0x8d00) {
+                code = 0x4800;
+            } else if((code & 0xff00) == 0x9100) {
+                code = 0x5000;
+            } else if((code & 0xff00) == 0x9200) {
+                code = 0x5200;
+            } else if((code & 0xff00) == 0x9300) {
+                code = 0x5300;
+            }
+        }
+    }
     /* Check for buffer Full */
     //TODO Maybe beeeeeeep or something although that should happend when internal buffer is full
     if (ttail==head) {
@@ -792,7 +820,7 @@ irq1_end:
         const auto flg = mem_readb(BIOS_KEYBOARD_FLAGS1);
         const auto ins = static_cast<bool>(flg & BIOS_KEYBOARD_FLAGS1_INSERT_ACTIVE);
         const auto ssl = static_cast<uint8_t>(ins ? CURSOR_SCAN_LINE_INSERT : CURSOR_SCAN_LINE_NORMAL);
-        if (CurMode->type == M_TEXT) {
+        if (CurMode->type == M_TEXT && inshell) {
             if (machine==MCH_MDA||machine==MCH_HERC)
                 INT10_SetCursorShape((ssl + 6) | (invisible?0x20:0x00), CURSOR_SCAN_LINE_END + 6); /* 14 - 8 = 6 */
             else
@@ -1352,6 +1380,45 @@ Bitu INT16_Handler(void) {
         /* Weird call used by some dos apps */
         LOG(LOG_BIOS,LOG_NORMAL)("INT16:55:Word TSR compatible call");
         break;
+    case 0xf0:
+        // J-3100 beep
+        break;
+    case 0xf1:
+        // J-3100 set shift
+        break;
+    case 0xf2:
+        // J-3100 key click sound
+        break;
+    case 0xf5:
+        // J-3100 set key data
+        if(J3_IsJapanese()) {
+            if(BIOS_AddKeyToBuffer(reg_bx)) {
+                reg_ax = 0x0000;
+            } else {
+                // buffer full
+                reg_ax = 0xffff;
+            }
+        }
+        break;
+    case 0xf6:
+        // J-3100 check key buffer
+        if(J3_IsJapanese()) {
+            uint16_t size, head, tail;
+            size = mem_readw(BIOS_KEYBOARD_BUFFER_END) - mem_readw(BIOS_KEYBOARD_BUFFER_START);
+            head = mem_readw(BIOS_KEYBOARD_BUFFER_HEAD);
+            tail = mem_readw(BIOS_KEYBOARD_BUFFER_TAIL);
+            reg_ax = 0;
+            if(tail > head) {
+                reg_ax = (tail - head) / 2;
+            } else if(tail < head) {
+                reg_ax = (tail + size - head) / 2;
+            }
+            if(reg_ax == size / 2) {
+                reg_ax = 0xffff;
+            }
+            reg_bx = J3_GetMachineCode() == 0 ? 0x6a74 : J3_GetMachineCode();
+        }
+        break;
     default:
         LOG(LOG_BIOS,LOG_ERROR)("INT16:Unhandled call %02X",reg_ah);
         break;
