@@ -742,9 +742,12 @@ uint8_t Mouse_GetButtonState(void) {
 }
 
 char text[5000];
+extern std::list<uint16_t> bdlist;
 extern bool isDBCSCP();
 extern std::vector<std::pair<int,int>> jtbs;
+extern std::map<int, int> pc98boxdrawmap;
 const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint16_t *textlen) {
+    bdlist = {};
 	uint16_t result=0, len=0;
 	uint8_t page = real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE);
 	uint16_t c=0, r=0;
@@ -790,6 +793,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                             text[len++]=result;
                         }
                         ReadCharAttr(rtl?ttf.cols-x-1:x,y,page,&result);
+                        if (curAC[rtl?ttf.cols-x-1:x].boxdraw||(!x&&curAC[rtl?ttf.cols-x:x+1].boxdraw)) bdlist.push_back(len);
                         text[len++]=result;
                         if ((int)x==c2&&c2<(int)(ttf.cols-1)&&curAC[rtl?ttf.cols-x-1:x].doublewide&&!curAC[rtl?ttf.cols-x:x+1].selected&&curAC[rtl?ttf.cols-x:x+1].skipped) {
                             ReadCharAttr(rtl?ttf.cols-x:x+1,y,page,&result);
@@ -797,10 +801,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                         }
                     }
                 while (len>0&&text[len-1]==32) text[--len]=0;
-                if (y<r2) {
-                    text[len++]='\r';
-                    text[len++]='\n';
-                }
+                if (y<r2) text[len++]='\n';
             }
             curAC += ttf.cols;
         }
@@ -808,7 +809,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
 #endif
 	for (int i=r1; i<=r2; i++) {
         bool lead1 = false, lead2 = false;
-        if (IS_DOSV && (CurMode->type != M_DCGA || IS_J3100)) {
+        if ((IS_DOSV && DOSV_CheckCJKVideoMode()) || (IS_J3100 && J3_IsJapanese())) {
             for (int k=0; k<c1; k++) {
                 if (lead1) lead1=false;
                 else if (isKanji1(real_readb(seg,(i*c+k)*2))) lead1=true;
@@ -819,7 +820,14 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
 				uint16_t address=((i*80)+(ttfuse&&rtl?ttfcols-j-1:j))*2;
 				PhysPt where = CurMode->pstart+address;
 				result=mem_readw(where);
-				if ((result & 0xFF00u) != 0u && (result & 0xFCu) != 0x08u && result==mem_readw(where+(ttfuse&&rtl?-2:2)) && ++j<c) {
+                if ((result & 0xFF00u) != 0u && (result & 0x7Cu) == 0x08u && (result%0xff) - (result/0x100) == 0xB) {
+                    uint8_t val = result/0x100+31;
+                    for (auto it = pc98boxdrawmap.begin(); it != pc98boxdrawmap.end(); ++it)
+                        if (it->second == val) {
+                            text[len++]=it->first;
+                            break;
+                        }
+                } else if ((result & 0xFF00u) != 0u && (result & 0x7Cu) != 0x08u && result==mem_readw(where+(ttfuse&&rtl?-2:2)) && ++j<c) {
 					result&=0x7F7F;
 					uint8_t j1=(result%0x100)+0x20, j2=result/0x100;
 					if (j1>32&&j1<127&&j2>32&&j2<127) {
@@ -834,7 +842,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                         text[len++]=result;
                 } else if (result)
                     text[len++]=result;
-            } else if (IS_DOSV && (CurMode->type != M_DCGA || IS_J3100)) {
+            } else if ((IS_DOSV && DOSV_CheckCJKVideoMode()) || (IS_J3100 && J3_IsJapanese())) {
                 if (lead2) lead2=false;
                 else if (isKanji1(real_readb(seg,(i*c+j)*2))) lead2=true;
                 result=real_readb(seg,(i*c+j)*2);
@@ -850,6 +858,12 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                 if (!isJEGAEnabled()||j>c1||!find) {
                     ReadCharAttr(ttfuse&&rtl?ttfcols-j-1:j,i,page,&result);
                     if (!result && CurMode->type == M_DCGA && !IS_J3100) result=32;
+#if defined(USE_TTF)
+                    if (ttfuse&&isDBCSCP()) {
+                        ttf_cell *curAC = curAttrChar+i*ttfcols;
+                        if (curAC[rtl?ttfcols-j-1:j].boxdraw||(!j&&curAC[rtl?ttf.cols-j:j+1].boxdraw)) bdlist.push_back(len);
+                    }
+#endif
                     text[len++]=result;
                     if (isJEGAEnabled() && find && del_flag && (text[len-1]&0xFF) == 0x7F) text[len-1]++;
                 }
@@ -861,10 +875,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
 			}
 		}
 		while (len>0&&text[len-1]==32) text[--len]=0;
-		if (i<r2) {
-			text[len++]='\r';
-			text[len++]='\n';
-		}
+		if (i<r2) text[len++]='\n';
 	}
     text[len] = 0;
     if (ttfuse&&rtl) std::reverse(text, text+len);
@@ -924,7 +935,7 @@ void Mouse_Select(int x1, int y1, int x2, int y2, int w, int h, bool select) {
 			if (IS_PC98_ARCH) {
 				PhysPt where = CurMode->pstart+((i*80)+j)*2;
 				mem_writeb(where+0x2000,mem_readb(where+0x2000)^16);
-			} else if (IS_DOSV && !IS_J3100) {
+			} else if ((IS_DOSV && DOSV_CheckCJKVideoMode()) || (IS_J3100 && J3_IsJapanese())) {
 				uint8_t attr = real_readb(seg,(i*c+j)*2+1);
 				real_writeb(seg,(i*c+j)*2+1,attr/0x10+(attr&0xF)*0x10);
 				if (j==c2) WriteCharTopView(c*i*2,j+1);
