@@ -31,10 +31,16 @@
 
 #include "config.h"
 #include "dosbox.h"
+#include "setup.h"
+#include "jfont.h"
 
 #include <SDL.h>
 #include "gui_tk.h"
 
+#include <math.h> /* floor */
+bool isDBCSCP();
+uint8_t *GetDbcs14Font(Bitu code, bool &is14);
+
 namespace GUI {
 
 /* start <= y < stop, region reserved for top level window title bar */
@@ -68,7 +74,7 @@ std::map<const char *,Font *,Font::ltstr> Font::registry;
 
 bool ToplevelWindow::mouseDoubleClicked(int x, int y, MouseButton button) {
 	if (button == Left && x < (6+titlebox_sysmenu_width) && x > 6 && y >= titlebar_y_start && y < titlebar_y_stop) {
-		systemMenu->executeAction("Close");
+		systemMenu->executeAction(MSG_Get("CLOSE"));
 		return true;
 	}
 	BorderedWindow::mouseClicked(x,y,button);
@@ -479,37 +485,54 @@ void Drawable::drawText(const Char c, bool interpret)
 	font->drawChar(this,c);
 }
 
+Char prvc = 0;
 void BitmapFont::drawChar(Drawable *d, const Char c) const {
-#define move(x) (ptr += ((x)+bit)/8-(((x)+bit)<0), bit = ((x)+bit+(((x)+bit)<0?8:0))%8)
-	const unsigned char *ptr = bitmap;
-	int bit = 0;
-
-	if (c > last) return;
-
-	if (char_position != NULL) {
-		ptr = char_position[c];
-		bit = 0;
-	} else {
-		move(character_step*((int)c));
-	}
-
+	const unsigned char *ptr = bitmap, *optr = ptr;
+	int bit = 0, i = 0, w = 0, h = 0;
+	bool is14 = false;
+	if (c > last) {prvc = 0;return;}
+    if (IS_PC98_ARCH || IS_JEGA_ARCH || isDBCSCP()) {
+        if (isKanji1(c) && prvc == 0) {
+            prvc = c;
+            return;
+        } else if (isKanji2(c) && prvc > 2) {
+            optr = GetDbcs14Font(prvc*0x100+c, is14);
+            prvc = is14?2:1;
+        } else if (prvc < 0x81)
+            prvc = 0;
+    } else
+        prvc = 0;
+#define move(x) (ptr += (((x)+bit)/8-(((x)+bit)<0))*(prvc==1||prvc==2?2:1), bit = ((x)+bit+(((x)+bit)<0?8:0))%8)
+	int ht = prvc==1?16:height, at = prvc==1?11:ascent;
 	int rs = row_step;
-	int w = (widths != NULL?widths[c]:width);
-	int h = (ascents != NULL?ascents[c]:height);
-	Drawable out(*d,d->getX(),d->getY()-ascent,w,h);
-
-	if (rs == 0) rs = isign(col_step)*w;
-	if (rs < 0) move(-rs*(h-1));
-	if (col_step < 0) move(abs(rs)-1);
-
-	for (int row = height-h; row < height; row++, move(rs-w*col_step)) {
-		for (int col = 0; col < w; col++, move(col_step)) {
-			if (!background_set != !(*ptr&(1<<bit)))
-				out.drawPixel(col,row);
+	for (i=0; i<(prvc?2:1); i++) {
+		if (prvc==1||prvc==2)
+			ptr = optr+i;
+		else if (char_position != NULL) {
+			ptr = char_position[i||!prvc?c:prvc];
+			bit = 0;
+		} else {
+			if (i) ptr = bitmap;
+			move(character_step*((int)(i||!prvc?c:prvc)));
 		}
+		w = widths!=NULL && prvc!=1 && prvc!=2 ? widths[i||!prvc?c:prvc] : width;
+		h = ascents!=NULL && prvc!=1 && prvc!=2 ? ascents[i||!prvc?c:prvc] : ht;
+        if (!i) {
+            if (rs == 0) rs = isign(col_step)*w;
+            if (rs < 0) move(-rs*(h-1));
+            if (col_step < 0) move(abs(rs)-1);
+        }
+		Drawable out(*d,d->getX(),d->getY()-at,w,h);
+		for (int row = ht-h; row < ht; row++, move(rs-w*col_step)) {
+			for (int col = 0; col < w; col++, move(col_step)) {
+				if (!background_set != !(*ptr&(1<<bit)))
+					out.drawPixel(col,row);
+			}
+		}
+		d->gotoXY(d->getX()+w,d->getY());
 	}
-	d->gotoXY(d->getX()+w,d->getY());
 #undef move
+	prvc = 0;
 }
 
 void Timer::check_to(unsigned int ticks) {
@@ -1219,7 +1242,11 @@ bool Input::keyDown(const Key &key)
 		break;
 	case Key::Backspace:
 		if (!key.shift && start_sel != end_sel) clearSelection();
-		else if (pos > 0) text.erase(text.begin()+int(--pos));
+		else if (pos > 0) {
+			text.erase(text.begin()+int(--pos));
+			if (start_sel > pos) start_sel = pos;
+			if (end_sel > pos) end_sel = pos;
+		}
 		break;
 	case Key::Delete:
 		if (key.shift) cutSelection();
@@ -1608,6 +1635,7 @@ void Screen::paint(Drawable &d) const
 
 unsigned int Screen::update(void *surface, unsigned int ticks)
 {
+    return 0; // FIXME
     (void)ticks;//UNUSED
 	paintAll(*buffer);
 	RGB *buf = buffer->buffer;
@@ -1918,15 +1946,64 @@ public:
 	    SDL_FreeSurface(surface);
 	}
 
-	void update(SDL_Surface *dest) const {
-	    SDL_BlitSurface(surface, NULL, dest, NULL);
-	}
+    static bool get_dest_pix
+    (   char* sp, SDL_PixelFormat *sf, SDL_PixelFormat *df, Uint32* pix )
+    {   Uint8 r, g, b, a;
+               SDL_GetRGBA( *(Uint32*)sp, sf, &r, &g, &b, &a );
+        *pix = SDL_MapRGBA( df, r, g, b, a );
+        return a>0;
+    }
+
+    void update(SDL_Surface *dest, int scale) const {
+        char            *dp, *sp, *sr0, *dr, *dr0;
+        int              x, y, v, h;
+        uint32_t         pix;
+        SDL_PixelFormat *fmt, *df;
+        bool             is_alpha;
+
+        // Direct blitting for pixel-to-pixel scale:
+        if( scale == 1 )
+        {   SDL_BlitSurface(surface, NULL, dest, NULL);
+            return;
+        }
+
+        fmt  = surface->format;
+        df   = dest   ->format;
+        SDL_LockSurface( dest    );
+        SDL_LockSurface( surface );
+        dr0 = (char*)dest   ->pixels;
+        sr0 = (char*)surface->pixels;
+
+        for( y = 0; y < surface->h; y += 1 )
+        {   dp = dr0;
+            sp = sr0;
+            for( x = 0; x < surface->w; x += 1 )
+            {   is_alpha = get_dest_pix( sp, fmt, df, &pix );
+                for( h = 0; h < scale; h += 1 )
+                {   if( is_alpha )
+                    {   memcpy( dp, &pix, df->BytesPerPixel );  }
+                    dp += df->BytesPerPixel;
+                }
+                sp += fmt->BytesPerPixel;
+            }
+            sr0 = sr0 + surface->pitch;
+            
+            dr = dr0 + dest->pitch;
+            for( v = 0; v < scale - 1; v += 1 )
+            {   memcpy( dr, dr0, scale * surface->w * dest->format->BytesPerPixel );
+                dr = dr + dest->pitch;
+            }
+            dr0 = dr;
+        }
+        SDL_UnlockSurface( dest    );
+        SDL_UnlockSurface( surface );
+    }
 };
 
 ScreenSDL::~ScreenSDL() {
 }
 
-ScreenSDL::ScreenSDL(SDL_Surface *surface) : Screen(new SDL_Drawable(surface->w, surface->h)), surface(surface), downx(0), downy(0), lastclick(0), lastdown(0) {
+ScreenSDL::ScreenSDL(SDL_Surface *surface, int scale) : Screen(new SDL_Drawable(surface->w/scale, surface->h/scale)), surface(surface), downx(0), downy(0), lastclick(0), lastdown(0), scale(scale) {
 	current_abs_time = start_abs_time = SDL_GetTicks();
 	current_time = 0;
 }
@@ -1935,7 +2012,7 @@ Ticks ScreenSDL::update(Ticks ticks)
 {
 	Timer::check_to(ticks);
 	paintAll(*buffer);
-	static_cast<SDL_Drawable*>(buffer)->update(surface);
+	static_cast<SDL_Drawable*>(buffer)->update(surface, scale);
 
 	return Timer::next();
 }
@@ -1950,7 +2027,7 @@ void ScreenSDL::paint(Drawable &d) const {
 	test(d);
 }
 
-bool ScreenSDL::event(const SDL_Event &event) {
+bool ScreenSDL::event(SDL_Event &event) {
 	bool rc;
 
 #if defined(C_SDL2)
@@ -2005,23 +2082,36 @@ bool ScreenSDL::event(const SDL_Event &event) {
 		if (key.special == GUI::Key::CapsLock || key.special == GUI::Key::NumLock) keyUp(key);
 		return rc;
 	}
-	case SDL_MOUSEMOTION:
-		if (event.motion.state) {
-			if (abs(event.motion.x-downx) <= 10 && abs(event.motion.y-downy) <= 10)
-				break;
-			downx = -11; downy = -11;
-			if (event.motion.state&SDL_BUTTON(1))
-				return mouseDragged(event.motion.x, event.motion.y, GUI::Left);
-			else if (event.motion.state&SDL_BUTTON(2))
-				return mouseDragged(event.motion.x, event.motion.y, GUI::Middle);
-			else if (event.motion.state&SDL_BUTTON(3))
-				return mouseDragged(event.motion.x, event.motion.y, GUI::Right);
-			break;
-		}
+    case SDL_MOUSEMOTION:
+    {
+        const auto m = event.motion;
+        const auto x = m.x / scale;
+        const auto y = m.y / scale;
 
-		return mouseMoved(event.motion.x, event.motion.y);
+        if (m.state)
+        {
+            if(abs(x - downx) <= 10 && abs(y - downy) <= 10)
+                break;
 
+            downx = -11;
+            downy = -11;
+
+            if(m.state & SDL_BUTTON(1))
+                return mouseDragged(x, y, Left);
+
+            if(m.state & SDL_BUTTON(2))
+                return mouseDragged(x, y, Middle);
+
+            if(m.state & SDL_BUTTON(3))
+                return mouseDragged(x, y, Right);
+
+            break;
+        }
+
+        return mouseMoved(x, y);
+    }
 	case SDL_MOUSEBUTTONDOWN:
+		event.button.x /= scale; event.button.y /= scale;
 		rc = mouseDown(event.button.x, event.button.y, SDL_to_GUI(event.button.button));
 		if (abs(event.button.x-downx) > 10 || abs(event.button.y-downy) > 10) lastclick = 0;
 		downx = event.button.x; downy = event.button.y;
@@ -2029,6 +2119,7 @@ bool ScreenSDL::event(const SDL_Event &event) {
 		return rc;
 
 	case SDL_MOUSEBUTTONUP:
+		event.button.x /= scale; event.button.y /= scale;
 		rc = mouseUp(event.button.x, event.button.y, SDL_to_GUI(event.button.button));
 		if (lastdown != 0 && abs(event.button.x-downx) < 10 && abs(event.button.y-downy) < 10) {
 			if (lastclick == 0 || (GUI::Timer::now()-lastclick) > 200) {
@@ -2050,6 +2141,109 @@ bool ScreenSDL::event(const SDL_Event &event) {
 	return false;
 }
 
+void WindowInWindow::paintScrollBar3DInset(Drawable &dscroll, int x, int y, int w, int h) const {
+    // Windows 3.1 renders the shadow one pixel wide, no highlight
+    dscroll.setColor(Color::Shadow3D);
+    dscroll.drawLine(x,y,x+w-1,y);
+    dscroll.drawLine(x,y,x,    y+h-1);
+}
+
+void WindowInWindow::paintScrollBar3DOutset(Drawable &dscroll, int x, int y, int w, int h) const {
+    dscroll.setColor(Color::Light3D);
+    dscroll.drawLine(x,y,x+w-2,y);
+    dscroll.drawLine(x,y,x,    y+h-2);
+
+    // Windows 3.1 renders the shadow two pixels wide
+    dscroll.setColor(Color::Shadow3D);
+    dscroll.drawLine(x,    y+h-1,x+w-1,y+h-1);
+    dscroll.drawLine(x+w-1,y,    x+w-1,y+h-1);
+
+    dscroll.drawLine(x+1,  y+h-2,x+w-1,y+h-2);
+    dscroll.drawLine(x+w-2,y+1,  x+w-2,y+h-2);
+}
+
+void WindowInWindow::paintScrollBarThumb(Drawable &dscroll, vscrollbarlayout &vsl) const {
+    // black border
+    dscroll.setColor(vsl.disabled ? Color::Shadow3D : Color::Black);
+    dscroll.drawRect(vsl.xleft,vsl.ytop,vsl.thumbwidth-1,vsl.thumbheight-1);
+
+    // 3D outset style, 1 pixel inward each side, inside the black rectangle we just drew
+    paintScrollBar3DOutset(dscroll, vsl.xleft+1, vsl.ytop+1, vsl.thumbwidth-2, vsl.thumbheight-2);
+}
+
+void WindowInWindow::paintScrollBarBackground(Drawable &dscroll,const vscrollbarlayout &vsl) const {
+    /* scroll bar border, background */
+    dscroll.setColor(vsl.disabled ? Color::Shadow3D : Color::Black);
+    dscroll.drawRect(vsl.scrollthumbRegion.x,  vsl.scrollthumbRegion.y,  vsl.scrollthumbRegion.w-1,vsl.scrollthumbRegion.h-1);
+
+    dscroll.setColor(Color::Background3D);
+    dscroll.fillRect(vsl.scrollthumbRegion.x+1,vsl.scrollthumbRegion.y+1,vsl.scrollthumbRegion.w-2,vsl.scrollthumbRegion.h-2);
+}
+
+void WindowInWindow::paintScrollBarThumbDragOutline(Drawable &dscroll,const vscrollbarlayout &vsl) const {
+    // Windows 3.1 also draws an inverted dotted rectangle around the thumb where it WOULD be
+    // before quantization to scroll position.
+    int x = 0;
+    int y = (drag_y - vsl.scrollthumbRegion.y) - ((vsl.thumbheight + 2) / 2);
+    if (y < 0) y = 0;
+    if (y > vsl.thumbtravel) y = vsl.thumbtravel;
+    dscroll.setColor(Color::Light3D);
+    dscroll.drawDotRect(x+vsl.scrollthumbRegion.x,y+vsl.scrollthumbRegion.y,vsl.thumbwidth-1,vsl.thumbheight-1);
+}
+
+void WindowInWindow::getVScrollInfo(vscrollbarlayout &vsl) const {
+    vsl.draw = vsl.drawthumb = vsl.disabled = false;
+    if (vscroll_display_width >= 4 && height >= 4) {
+        vsl.disabled = (scroll_pos_h == 0);
+
+        vsl.draw = true;
+        vsl.scrollbarRegion.x = width - vscroll_display_width;
+        vsl.scrollbarRegion.y = 0;
+        vsl.scrollbarRegion.w = vscroll_display_width;
+        vsl.scrollbarRegion.h = height;
+
+        /* the "thumb". make it fixed size, Windows 3.1 style.
+         * this code could adapt to the more range-aware visual style of Windows 95 later. */
+        vsl.thumbwidth = vsl.scrollbarRegion.w;
+        vsl.thumbheight = vsl.scrollbarRegion.w;
+
+        vsl.scrollthumbRegion.x = 0; /* relative to scrollbarRegion */
+        vsl.scrollthumbRegion.y = vsl.thumbheight - 1;/* relative to scrollbarRegion */
+        vsl.scrollthumbRegion.w = vsl.scrollbarRegion.w;
+        vsl.scrollthumbRegion.h = vsl.scrollbarRegion.h - ((vsl.thumbheight - 1) * 2);
+
+        vsl.thumbtravel = vsl.scrollthumbRegion.h - vsl.thumbheight;
+        if (vsl.thumbtravel < 0) vsl.thumbtravel = 0;
+        vsl.drawthumb = (vsl.thumbheight <= vsl.scrollthumbRegion.h) && (!vsl.disabled) && (scroll_pos_h > 0);
+
+        if (vsl.drawthumb) {
+            vsl.ytop = ((vsl.thumbtravel * scroll_pos_y) / scroll_pos_h) + vsl.scrollthumbRegion.y;
+            vsl.xleft = vsl.scrollthumbRegion.x;
+        }
+        else {
+            vsl.thumbwidth = vsl.thumbheight = vsl.thumbtravel = 0;
+        }
+    }
+}
+
+void WindowInWindow::paintScrollBarArrowInBox(Drawable &dscroll,const int x,const int y,const int w,const int h,bool downArrow,bool disabled) const {
+    const int aw = 7;
+    const int ah = 7;
+    const int ax = ((w - aw) / 2) + x;
+    const int ay = ((h - ah) / 2) + y;
+
+    dscroll.setColor(disabled ? Color::Shadow3D : Color::Black);
+
+    if (downArrow) {
+        dscroll.fillRect(ax+2,ay,3,3);
+        for (int c=0;c <= 3;c++) dscroll.drawLine(ax+3-(3-c), ay+3+c, ax+3+(3-c), ay+3+c);
+    }
+    else {
+        for (int c=0;c <= 3;c++) dscroll.drawLine(ax+3-c, ay+c, ax+3+c, ay+c);
+        dscroll.fillRect(ax+2,ay+4,3,3);
+    }
+}
+
 void WindowInWindow::paintAll(Drawable &d) const {
     int xadj = -scroll_pos_x;
     int yadj = -scroll_pos_y;
@@ -2084,78 +2278,181 @@ void WindowInWindow::paintAll(Drawable &d) const {
         dchild.drawLine(w,0,w,h);
     }
 
-    if (vscroll && vscroll_display_width >= 4) {
-        // TODO: Need a vertical scrollbar window object
+    if (vscroll) {
+        vscrollbarlayout vsl;
 
-        Drawable dscroll(d,width - vscroll_display_width,0,vscroll_display_width,height);
+        getVScrollInfo(vsl);
+        if (vsl.draw) {
+            Drawable dscroll(d,vsl.scrollbarRegion.x,vsl.scrollbarRegion.y,vsl.scrollbarRegion.w,vsl.scrollbarRegion.h);
 
-        bool disabled = (scroll_pos_h == 0);
+            paintScrollBarBackground(dscroll,vsl);
 
-        /* scroll bar border, gray background */
-        dscroll.setColor(disabled ? Color::Shadow3D : Color::Black);
-        dscroll.drawRect(0,0,vscroll_display_width-1,height-1);
+            /* up arrow */
+            {
+                const int x = 0;
+                const int y = 0;
+                const int w = vsl.scrollbarRegion.w;
+                const int h = vsl.scrollthumbRegion.y + 1; /* want black border of thumb to overlap our black border 1 pixel */
 
-        dscroll.setColor(Color::Background3D);
-        dscroll.fillRect(1,1,vscroll_display_width-2,height-2);
+                // black border
+                dscroll.setColor(vsl.disabled ? Color::Shadow3D : Color::Black);
+                dscroll.drawRect(x,y,w-1,h-1);
 
-        /* the "thumb". make it fixed size, Windows 3.1 style.
-         * this code could adapt to the more range-aware visual style of Windows 95 later. */
-        int thumbwidth = vscroll_display_width - 2;
-        int thumbheight = vscroll_display_width - 2;
-        int thumbtravel = height - 2 - thumbheight;
-        if (thumbtravel < 0) thumbtravel = 0;
-        int ytop = 1 + ((scroll_pos_h > 0) ?
-            ((thumbtravel * scroll_pos_y) / scroll_pos_h) :
-            0);
+                // 3D outset style, 1 pixel inward each side, inside the black rectangle we just drew
+                if (vscroll_uparrowhold && vscroll_uparrowdown)
+                    paintScrollBar3DInset(dscroll,x+1,y+1,w-2,h-2);
+                else
+                    paintScrollBar3DOutset(dscroll,x+1,y+1,w-2,h-2);
 
-        if (thumbheight <= (height + 2) && !disabled) {
-            int xleft = 1;
-            dscroll.setColor(Color::Light3D);
-            dscroll.drawLine(xleft,ytop,xleft+thumbwidth-1,ytop);
-            dscroll.drawLine(xleft,ytop,xleft,ytop+thumbheight-1);
+                // up arrow
+                const int nudge = ((vscroll_uparrowhold && vscroll_uparrowdown) ? 1 : 0);
+                paintScrollBarArrowInBox(dscroll,x+nudge,y+nudge,w,h,/*downArrow*/false,vsl.disabled);
+            }
 
-            // Windows 3.1 renders the shadow two pixels wide
-            dscroll.setColor(Color::Shadow3D);
-            dscroll.drawLine(xleft,ytop+thumbheight-1,xleft+thumbwidth-1,ytop+thumbheight-1);
-            dscroll.drawLine(xleft+thumbwidth-1,ytop,xleft+thumbwidth-1,ytop+thumbheight-1);
+            /* down arrow */
+            {
+                const int x = 0;
+                const int y = vsl.scrollthumbRegion.y + vsl.scrollthumbRegion.h - 1; /* want black border of thumb to overlap our black border 1 pixel */
+                const int w = vsl.scrollbarRegion.w;
+                const int h = height - y;
 
-            dscroll.drawLine(xleft+1,ytop+thumbheight-2,xleft+thumbwidth-2,ytop+thumbheight-2);
-            dscroll.drawLine(xleft+thumbwidth-2,ytop+1,xleft+thumbwidth-2,ytop+thumbheight-2);
+                // black border
+                dscroll.setColor(vsl.disabled ? Color::Shadow3D : Color::Black);
+                dscroll.drawRect(x,y,w-1,h-1);
 
-            // Windows 3.1 also draws a hard black line around the thumb that can coincide with the border
-            dscroll.setColor(Color::Black);
-            dscroll.drawLine(xleft,ytop-1,xleft+thumbwidth-1,ytop-1);
-            dscroll.drawLine(xleft,ytop+thumbheight,xleft+thumbwidth-1,ytop+thumbheight);
+                // 3D outset style, 1 pixel inward each side, inside the black rectangle we just drew
+                if (vscroll_downarrowhold && vscroll_downarrowdown)
+                    paintScrollBar3DInset(dscroll,x+1,y+1,w-2,h-2);
+                else
+                    paintScrollBar3DOutset(dscroll,x+1,y+1,w-2,h-2);
 
-            // Windows 3.1 also draws an inverted dotted rectangle around the thumb where it WOULD be
-            // before quantization to scroll position.
-            if (vscroll_dragging) {
-                xleft = 0;
-                ytop = drag_y - ((thumbheight + 2) / 2);
-                if (ytop < 0) ytop = 0;
-                if (ytop > thumbtravel) ytop = thumbtravel;
-                dscroll.setColor(Color::Light3D);
-                dscroll.drawDotRect(xleft,ytop,thumbwidth+1,thumbheight+1);
+                // down arrow
+                const int nudge = ((vscroll_downarrowhold && vscroll_downarrowdown) ? 1 : 0);
+                paintScrollBarArrowInBox(dscroll,x+nudge,y+nudge,w,h,/*downArrow*/true,vsl.disabled);
+            }
+
+            if (vsl.drawthumb) {
+                paintScrollBarThumb(dscroll,vsl);
+                if (vscroll_dragging) paintScrollBarThumbDragOutline(dscroll,vsl);
             }
         }
     }
 }
 
+static const Ticks vscroll_holdwait = 150;
+static const Ticks vscroll_unitstep = 50;
+static const Ticks vscroll_unitinit = 50;
+static const Ticks vscroll_unitpersec = 600;
+
+Ticks WindowInWindow::DragTimer_Callback::timerExpired(Ticks time) {
+    (void)time;//UNUSED
+    if (wnd != NULL) {
+        if (wnd->vscroll_downarrowhold) {
+            Ticks tdelta = Timer::now() - wnd->drag_start;
+            if (tdelta >= vscroll_holdwait) {
+                tdelta -= vscroll_holdwait;
+
+                const int pdelta = (int)((tdelta * vscroll_unitpersec) / 1000) + vscroll_unitinit;
+
+                wnd->scroll_pos_y = wnd->drag_start_pos + pdelta;
+                if (wnd->scroll_pos_y > wnd->scroll_pos_h) {
+                    wnd->scroll_pos_y = wnd->scroll_pos_h;
+                    return 0;
+                }
+                if (wnd->scroll_pos_y < 0) {
+                    wnd->scroll_pos_y = 0;
+                    return 0;
+                }
+            }
+
+            return 55;
+        }
+        if (wnd->vscroll_uparrowhold) {
+            Ticks tdelta = Timer::now() - wnd->drag_start;
+            if (tdelta >= vscroll_holdwait) {
+                tdelta -= vscroll_holdwait;
+
+                const int pdelta = (int)((tdelta * vscroll_unitpersec) / 1000) + vscroll_unitinit;
+
+                wnd->scroll_pos_y = wnd->drag_start_pos - pdelta;
+                if (wnd->scroll_pos_y > wnd->scroll_pos_h) {
+                    wnd->scroll_pos_y = wnd->scroll_pos_h;
+                    return 0;
+                }
+                if (wnd->scroll_pos_y < 0) {
+                    wnd->scroll_pos_y = 0;
+                    return 0;
+                }
+            }
+
+            return 55;
+        }
+    }
+
+    return 0;
+}
+
 bool WindowInWindow::mouseDragged(int x, int y, MouseButton button)
 {
-    if (vscroll_dragging) {
-        int thumbheight = vscroll_display_width - 2;
-        int thumbtravel = height - 2 - thumbheight;
-        if (thumbtravel < 0) thumbtravel = 0;
+    if (vscroll_uparrowhold) {
+        vscrollbarlayout vsl;
 
-        double npos = (double(y - 1 - (thumbheight / 2)) * scroll_pos_h) / thumbtravel;
+        getVScrollInfo(vsl);
+
+        if (x >= vsl.scrollbarRegion.x && x < (vsl.scrollbarRegion.x+vsl.scrollbarRegion.w) && y >= 0 && y < vsl.scrollthumbRegion.y) {
+            if (!vscroll_uparrowdown) {
+                vscroll_uparrowdown = true;
+                drag_start = Timer::now();
+                drag_start_pos = scroll_pos_y;
+
+                drag_timer_cb.wnd = this;
+                drag_timer.remove(&drag_timer_cb);
+                drag_timer.add(&drag_timer_cb,55);
+            }
+        }
+        else {
+            vscroll_uparrowdown = false;
+            drag_timer.remove(&drag_timer_cb);
+        }
+
+        return true;
+    }
+    if (vscroll_downarrowhold) {
+        vscrollbarlayout vsl;
+
+        getVScrollInfo(vsl);
+
+        if (x >= vsl.scrollbarRegion.x && x < (vsl.scrollbarRegion.x+vsl.scrollbarRegion.w) && y >= (vsl.scrollthumbRegion.y+vsl.scrollthumbRegion.h) && y < height) {
+            if (!vscroll_downarrowdown) {
+                vscroll_downarrowdown = true;
+                drag_start = Timer::now();
+                drag_start_pos = scroll_pos_y;
+
+                drag_timer_cb.wnd = this;
+                drag_timer.remove(&drag_timer_cb);
+                drag_timer.add(&drag_timer_cb,55);
+            }
+        }
+        else {
+            vscroll_downarrowdown = false;
+            drag_timer.remove(&drag_timer_cb);
+        }
+
+        return true;
+    }
+    if (vscroll_dragging) {
+        vscrollbarlayout vsl;
+
+        getVScrollInfo(vsl);
+
+        drag_x = x;
+        drag_y = y;
+
+        double npos = (double(y - vsl.scrollthumbRegion.y - (vsl.thumbheight / 2)) * scroll_pos_h) / vsl.thumbtravel;
         int nipos = int(floor(npos + 0.5));
         if (nipos < 0) nipos = 0;
         if (nipos > scroll_pos_h) nipos = scroll_pos_h;
         scroll_pos_y = nipos;
-
-        drag_x = x;
-        drag_y = y;
         return true;
     }
 
@@ -2185,20 +2482,48 @@ bool WindowInWindow::mouseDragged(int x, int y, MouseButton button)
 bool WindowInWindow::mouseDown(int x, int y, MouseButton button)
 {
     if (vscroll && x >= (width - vscroll_display_width) && button == GUI::Left) {
-        mouseChild = this;
-        vscroll_dragging = true;
-        drag_x = x;
-        drag_y = y;
+        vscrollbarlayout vsl;
 
-        int thumbheight = vscroll_display_width - 2;
-        int thumbtravel = height - 2 - thumbheight;
-        if (thumbtravel < 0) thumbtravel = 0;
+        getVScrollInfo(vsl);
 
-        double npos = (double(y - 1 - (thumbheight / 2)) * scroll_pos_h) / thumbtravel;
-        int nipos = int(floor(npos + 0.5));
-        if (nipos < 0) nipos = 0;
-        if (nipos > scroll_pos_h) nipos = scroll_pos_h;
-        scroll_pos_y = nipos;
+        if (y < vsl.scrollthumbRegion.y) {
+            vscroll_uparrowdown = true;
+            vscroll_uparrowhold = true;
+            drag_start_pos = scroll_pos_y;
+            drag_start = Timer::now();
+            mouseChild = this;
+            drag_x = x;
+            drag_y = y;
+
+            drag_timer_cb.wnd = this;
+            drag_timer.remove(&drag_timer_cb);
+            drag_timer.add(&drag_timer_cb,55);
+        }
+        else if (y >= (vsl.scrollthumbRegion.y+vsl.scrollthumbRegion.h)) {
+            vscroll_downarrowdown = true;
+            vscroll_downarrowhold = true;
+            drag_start_pos = scroll_pos_y;
+            drag_start = Timer::now();
+            mouseChild = this;
+            drag_x = x;
+            drag_y = y;
+
+            drag_timer_cb.wnd = this;
+            drag_timer.remove(&drag_timer_cb);
+            drag_timer.add(&drag_timer_cb,55);
+        }
+        else if (y >= vsl.scrollthumbRegion.y && y < (vsl.scrollthumbRegion.y+vsl.scrollthumbRegion.h)) {
+            vscroll_dragging = true;
+            mouseChild = this;
+            drag_x = x;
+            drag_y = y;
+
+            double npos = (double(y - vsl.scrollthumbRegion.y - (vsl.thumbheight / 2)) * scroll_pos_h) / vsl.thumbtravel;
+            int nipos = int(floor(npos + 0.5));
+            if (nipos < 0) nipos = 0;
+            if (nipos > scroll_pos_h) nipos = scroll_pos_h;
+            scroll_pos_y = nipos;
+        }
 
         return true;
     }
@@ -2240,12 +2565,41 @@ bool WindowInWindow::mouseDown(int x, int y, MouseButton button)
 
 bool WindowInWindow::mouseUp(int x, int y, MouseButton button)
 {
-    if (vscroll_dragging) {
-        int thumbheight = vscroll_display_width - 2;
-        int thumbtravel = height - 2 - thumbheight;
-        if (thumbtravel < 0) thumbtravel = 0;
+    if (vscroll_uparrowhold) {
+        drag_timer.remove(&drag_timer_cb);
+        vscroll_uparrowdown = false;
+        vscroll_uparrowhold = false;
+        mouseChild = NULL;
 
-        double npos = (double(y - 1 - (thumbheight / 2)) * scroll_pos_h) / thumbtravel;
+        if (scroll_pos_y == drag_start_pos) {
+            if ((scroll_pos_y -= vscroll_unitstep) < 0)
+                scroll_pos_y = 0;
+        }
+
+        return true;
+    }
+    if (vscroll_downarrowhold) {
+        drag_timer.remove(&drag_timer_cb);
+        vscroll_downarrowdown = false;
+        vscroll_downarrowhold = false;
+        mouseChild = NULL;
+
+        if (scroll_pos_y == drag_start_pos) {
+            if ((scroll_pos_y += vscroll_unitstep) > scroll_pos_h)
+                scroll_pos_y = scroll_pos_h;
+        }
+
+        return true;
+    }
+    if (vscroll_dragging) {
+        vscrollbarlayout vsl;
+
+        getVScrollInfo(vsl);
+
+        drag_x = x;
+        drag_y = y;
+
+        double npos = (double(y - vsl.scrollthumbRegion.y - (vsl.thumbheight / 2)) * scroll_pos_h) / vsl.thumbtravel;
         int nipos = int(floor(npos + 0.5));
         if (nipos < 0) nipos = 0;
         if (nipos > scroll_pos_h) nipos = scroll_pos_h;
