@@ -33,7 +33,6 @@
 #include "callback.h"
 #include "dos_inc.h"
 #include "support.h"
-#include "builtin.h"
 #include "mapper.h"
 #include "render.h"
 #include "jfont.h"
@@ -58,8 +57,9 @@ extern bool addovl, addipx, addne2k, enableime;
 extern const char* RunningProgram;
 extern int enablelfn, msgcodepage;
 extern uint16_t countryNo;
-bool usecon = true;
-bool shellrun = false;
+extern unsigned int dosbox_shell_env_size;
+bool outcon = true, usecon = true;
+bool shellrun = false, prepared = false;
 
 uint16_t shell_psp = 0;
 Bitu call_int2e = 0;
@@ -84,7 +84,7 @@ static Bitu shellstop_handler(void) {
 	return CBRET_STOP;
 }
 
-static void SHELL_ProgramStart(Program * * make) {
+void SHELL_ProgramStart(Program * * make) {
 	*make = new DOS_Shell;
 }
 //Repeat it with the correct type, could do it in the function below, but this way it should be 
@@ -93,12 +93,10 @@ static void SHELL_ProgramStart_First_shell(DOS_Shell * * make) {
 	*make = new DOS_Shell;
 }
 
-#define CONFIG_SIZE 4096
-#define AUTOEXEC_SIZE 4096
 bool i4dos=false;
-static char i4dos_data[CONFIG_SIZE] = { 0 };
-static char config_data[CONFIG_SIZE] = { 0 };
-static char autoexec_data[AUTOEXEC_SIZE] = { 0 };
+char i4dos_data[CONFIG_SIZE] = { 0 };
+char config_data[CONFIG_SIZE] = { 0 };
+char autoexec_data[AUTOEXEC_SIZE] = { 0 };
 static std::list<std::string> autoexec_strings;
 typedef std::list<std::string>::iterator auto_it;
 
@@ -107,7 +105,9 @@ void runRescan(const char *str), DOSBox_SetSysMenu(void);
 void toSetCodePage(DOS_Shell *shell, int newCP, int opt);
 
 #if defined(WIN32)
-void MountAllDrives(Program * program, bool quiet) {
+void MountAllDrives(bool quiet) {
+    char str[100];
+    uint16_t n = 0;
     uint32_t drives = GetLogicalDrives();
     char name[4]="A:\\";
     for (int i=0; i<25; i++) {
@@ -116,7 +116,11 @@ void MountAllDrives(Program * program, bool quiet) {
             name[0]='A'+i;
             int type=GetDriveType(name);
             if (type!=DRIVE_NO_ROOT_DIR) {
-                if (!quiet) program->WriteOut("Mounting %c: => %s..\n", name[0], name);
+                if (!quiet) {
+                    sprintf(str, "Mounting %c: => %s..\r\n", name[0], name);
+                    n = (uint16_t)strlen(str);
+                    DOS_WriteFile(STDOUT,(uint8_t *)str,&n);
+                }
                 char mountstring[DOS_PATHLENGTH+CROSS_LEN+20];
                 name[2]=' ';
                 strcpy(mountstring,name);
@@ -124,8 +128,19 @@ void MountAllDrives(Program * program, bool quiet) {
                 strcat(mountstring,name);
                 strcat(mountstring," -Q");
                 runMount(mountstring);
-                if (!Drives[i] && !quiet) program->WriteOut("Drive %c: failed to mount.\n",name[0]);
-                else if(mountwarning && !quiet && type==DRIVE_FIXED && (strcasecmp(name,"C:\\")==0)) program->WriteOut(MSG_Get("PROGRAM_MOUNT_WARNING_WIN"));
+                if (!Drives[i] && !quiet) {
+                    sprintf(str, "Drive %c: failed to mount.\r\n",name[0]);
+                    n = (uint16_t)strlen(str);
+                    DOS_WriteFile(STDOUT,(uint8_t *)str,&n);
+                } else if(mountwarning && !quiet && type==DRIVE_FIXED && (strcasecmp(name,"C:\\")==0)) {
+                    strcpy(str, MSG_Get("PROGRAM_MOUNT_WARNING_WIN"));
+                    if (strlen(str)>2&&str[strlen(str)-1]=='\n'&&str[strlen(str)-2]!='\r') {
+                        str[strlen(str)-1]=0;
+                        strcat(str, "\r\n");
+                    }
+                    n = (uint16_t)strlen(str);
+                    DOS_WriteFile(STDOUT,(uint8_t *)str,&n);
+                }
             }
         }
     }
@@ -380,8 +395,11 @@ void DOS_Shell::ParseLine(char * line) {
 			LOG_MSG("SHELL:Redirect input from %s",in);
 			if(normalstdin) DOS_CloseFile(0);	//Close stdin
 			DOS_OpenFile(in,OPEN_READ,&dummy);	//Open new stdin
-		} else
+		} else {
 			WriteOut(!*in?"File open error\n":(dos.errorcode==DOSERR_ACCESS_DENIED?MSG_Get("SHELL_CMD_FILE_ACCESS_DENIED"):"File open error - %s\n"), in);
+			in = 0;
+			return;
+		}
 	}
 	bool fail=false;
 	char pipetmp[270];
@@ -443,6 +461,7 @@ void DOS_Shell::ParseLine(char * line) {
 	int Reflect_Menu(void);
 	Reflect_Menu();
 #endif
+	if (toc||(!toc&&((out&&DOS_FindDevice(out)!=DOS_FindDevice("con"))))) outcon=false;
 	if (toc||(!toc&&((out&&DOS_FindDevice(out)!=DOS_FindDevice("con"))||(in&&DOS_FindDevice(in)!=DOS_FindDevice("con"))))) usecon=false;
 
 	DoCommand(line);
@@ -484,7 +503,7 @@ void DOS_Shell::ParseLine(char * line) {
 		free(toc);
 		if (DOS_FindFirst(pipetmp, ~DOS_ATTR_VOLUME)) DOS_UnlinkFile(pipetmp);
 		}
-	usecon=true;
+	outcon=usecon=true;
 }
 
 
@@ -683,7 +702,7 @@ void DOS_Shell::Prepare(void) {
             if (IS_PC98_ARCH || IS_JEGA_ARCH)
                 countryNo = 81;
             else if (IS_DOSV)
-                countryNo = IS_PDOSV?86:(IS_CDOSV?886:(IS_KDOSV?82:81));
+                countryNo = IS_PDOSV?86:(IS_TDOSV?886:(IS_KDOSV?82:81));
 #if defined(WIN32)
 			else if (GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_ICOUNTRY, buffer, 128)) {
 				countryNo = uint16_t(atoi(buffer));
@@ -719,10 +738,10 @@ void DOS_Shell::Prepare(void) {
                     if (!newCP && IS_DOSV) {
                         if (IS_JDOSV) newCP=932;
                         else if (IS_PDOSV) newCP=936;
-                        else if (IS_CDOSV) newCP=949;
-                        else if (IS_KDOSV) newCP=950;
+                        else if (IS_KDOSV) newCP=949;
+                        else if (IS_TDOSV) newCP=950;
                     }
-                    if (newCP==932||newCP==936||newCP==949||newCP==950) {
+                    if (newCP==932||newCP==936||newCP==949||newCP==950||newCP==951) {
                         dos.loaded_codepage=newCP;
                         SetupDBCSTable();
                         runRescan("-A -Q");
@@ -803,7 +822,7 @@ void DOS_Shell::Prepare(void) {
 		{
 			const Section_prop* sec = static_cast<Section_prop*>(control->GetSection("dos"));
 			const char *automountstr = sec->Get_string("automountall");
-			if (strcmp(automountstr, "0") && strcmp(automountstr, "false")) MountAllDrives(this, !strcmp(automountstr, "quiet")||control->opt_fastlaunch);
+			if (strcmp(automountstr, "0") && strcmp(automountstr, "false")) MountAllDrives(!strcmp(automountstr, "quiet")||control->opt_fastlaunch);
 		}
 #endif
 		strcpy(i4dos_data, "");
@@ -922,15 +941,18 @@ public:
         int cp=dos.loaded_codepage;
         InitCodePage();
         force_conversion = false;
+        int ind=0;
 
         /* The user may have given .BAT files to run on the command line */
         if (!control->auto_bat_additional.empty()) {
             std::string cmd = "@echo off\n";
 
             for (unsigned int i=0;i<control->auto_bat_additional.size();i++) {
+                if (!control->opt_prerun) cmd += "\n";
                 if (!strncmp(control->auto_bat_additional[i].c_str(), "@mount c: ", 10)) {
                     cmd += control->auto_bat_additional[i]+"\n";
-                    cmd += "@c:";
+                    cmd += "@config -get lastmount>nul\n";
+                    cmd += "@if not '%CONFIG%'=='' %CONFIG%";
                 } else {
                     std::string batname;
                     //LOG_MSG("auto_bat_additional %s\n", control->auto_bat_additional[i].c_str());
@@ -945,16 +967,20 @@ public:
                     }
                     if(pos == std::string::npos) {  //Only a filename, mount current directory
                         batname = control->auto_bat_additional[i];
-                        cmd += "@mount c: . -q\n";
+                        cmd += "@mount c: . -nl -q\n";
                     } else { //Parse the path of .BAT file
                         std::string batpath = control->auto_bat_additional[i].substr(0,pos+1);
                         if (batpath==".\\") batpath=".";
                         else if (batpath=="..\\") batpath="..";
                         batname = control->auto_bat_additional[i].substr(pos+1);
-                        cmd += "@mount c: \"" + batpath + "\" -q\n";
+                        cmd += "@mount c: \"" + batpath + "\" -nl -q\n";
                     }
+                    std::string opt = control->opt_o.size() > ind && control->opt_o[ind].size() ? " "+control->opt_o[ind] : "";
+                    ind++;
                     bool templfn=!uselfn&&filename_not_8x3(batname.c_str())&&(enablelfn==-1||enablelfn==-2);
-                    cmd += "@c:\n";
+                    cmd += "@config -get lastmount>nul\n";
+                    cmd += "@set LASTMOUNT=%CONFIG%\n";
+                    cmd += "@if not '%LASTMOUNT%'=='' %LASTMOUNT%\n";
                     cmd += "@cd \\\n";
                     if (templfn) cmd += "@config -set lfn=true\n";
 #if defined(WIN32) && !defined(HX_DOS)
@@ -962,12 +988,13 @@ public:
 #endif
                     cmd += "@CALL \"";
                     cmd += batname;
-                    cmd += "\"\n";
+                    cmd += "\"" + opt + "\n";
                     if (templfn) cmd += "@config -set lfn=" + std::string(enablelfn==-1?"auto":"autostart") + "\n";
 #if defined(WIN32) && !defined(HX_DOS)
                     if (!winautorun) cmd += "@config -set startcmd=false\n";
 #endif
-                    cmd += "@mount c: -q -u";
+                    cmd += "@if not '%LASTMOUNT%'=='' mount %LASTMOUNT% -q -u\n";
+                    cmd += "@set LASTMOUNT=";
                 }
                 if (control->opt_prerun) cmd += "\n";
             }
@@ -1176,10 +1203,6 @@ static Bitu INT2E_Handler(void) {
 	return CBRET_NONE;
 }
 
-extern unsigned int dosbox_shell_env_size;
-void IPXNET_ProgramStart(Program * * make);
-void drivezRegister(std::string path, std::string dir);
-
 /* TODO: Why is all this DOS kernel and VFILE registration here in SHELL_Init()?
  *       That's like claiming that DOS memory and device initialization happens from COMMAND.COM!
  *       We need to move the DOS kernel initialization into another function, and the VFILE
@@ -1410,6 +1433,8 @@ void SHELL_Init() {
 	MSG_Add("SHELL_CMD_GOTO_HELP_LONG","GOTO label\n\n"
 		   "   label   Specifies a text string used in the batch program as a label.\n\n"
 		   "You type a label on a line by itself, beginning with a colon.\n");
+	MSG_Add("SHELL_CMD_HISTORY_HELP","Displays or clears the command history list.\n");
+	MSG_Add("SHELL_CMD_HISTORY_HELP_LONG","HISTORY [/C]\n\n  /C  Clears the command history list.\n");
 	MSG_Add("SHELL_CMD_SHIFT_HELP","Changes the position of replaceable parameters in a batch file.\n");
 	MSG_Add("SHELL_CMD_SHIFT_HELP_LONG","SHIFT\n");
 	MSG_Add("SHELL_CMD_FOR_HELP","Runs a specified command for each file in a set of files.\n");
@@ -1527,13 +1552,15 @@ void SHELL_Init() {
 		   "  $$   $ (dollar sign)\n");
     MSG_Add("SHELL_CMD_ALIAS_HELP", "Defines or displays aliases.\n");
     MSG_Add("SHELL_CMD_ALIAS_HELP_LONG", "ALIAS [name[=value] ... ]\n\nType ALIAS without parameters to display the list of aliases in the form:\n`ALIAS NAME = VALUE'\n");
+    MSG_Add("SHELL_CMD_ASSOC_HELP", "Displays or changes file extension associations.\n");
+    MSG_Add("SHELL_CMD_ASSOC_HELP_LONG", "ASSOC [.ext[=command] ... ]\n\nType ASSOC without parameters to display the current file associations.\nFile extensions must start with a dot (.); wildcards (* and ?) are allowed.\n");
 	MSG_Add("SHELL_CMD_CHCP_HELP", "Displays or changes the current DOS code page.\n");
-	MSG_Add("SHELL_CMD_CHCP_HELP_LONG", "CHCP [nnn [file]]\n\n  nnn   Specifies a code page number.\n  file  Specifies a code page file.\n\nSupported code pages for changing in the TrueType font output:\n\n437,808,850,852,853,855,857,858,860-866,869,872,874,1250-1258\n\nAlso double-byte code pages including 932, 936, 949, and 950.\n");
+	MSG_Add("SHELL_CMD_CHCP_HELP_LONG", "CHCP [nnn [file]]\n\n  nnn   Specifies a code page number.\n  file  Specifies a code page file.\n\nSupported code pages for changing in the TrueType font output:\n\n437,737,775,808,850,852,853,855,857,858,860-866,869,872,874\n\nWindows code pages: 1250,1251,1252,1253,1254,1255,1256,1257,1258\n\nAlso double-byte code pages including 932, 936, 949, and 950.\n");
 	MSG_Add("SHELL_CMD_CHCP_ACTIVE", "Active code page: %d\n");
 	MSG_Add("SHELL_CMD_CHCP_MISSING", "ASCII characters not defined in TTF font: %d\n");
 	MSG_Add("SHELL_CMD_CHCP_INVALID", "Invalid code page number - %s\n");
 	MSG_Add("SHELL_CMD_COUNTRY_HELP", "Displays or changes the current country.\n");
-	MSG_Add("SHELL_CMD_COUNTRY_HELP_LONG", "COUNTRY [nnn] \n\n  nnn   Specifies a country code.\n\nDate and time formats will be affected by the specified country code.\n");
+	MSG_Add("SHELL_CMD_COUNTRY_HELP_LONG", "COUNTRY [nnn] \n\n  nnn   Specifies a country code.\n\nCountry-specific information such as date and time formats will be affected.\n");
     MSG_Add("SHELL_CMD_CTTY_HELP","Changes the terminal device used to control the system.\n");
 	MSG_Add("SHELL_CMD_CTTY_HELP_LONG","CTTY device\n  device        The terminal device to use, such as CON.\n");
 	MSG_Add("SHELL_CMD_MORE_HELP","Displays output one screen at a time.\n");
@@ -1556,7 +1583,6 @@ void SHELL_Init() {
 	reg_ip=RealOff(newcsip);
 
 	CALLBACK_Setup(call_shellstop,shellstop_handler,CB_IRET,"shell stop");
-	PROGRAMS_MakeFile("COMMAND.COM",SHELL_ProgramStart);
 
     /* NTS: Some DOS programs behave badly if run from a command interpreter
      *      who's PSP segment is too low in memory and does not appear in
@@ -1668,119 +1694,6 @@ void SHELL_Init() {
 	extern bool Mouse_Drv;
 	Mouse_Drv = true;
 
-    std::string dirname="drivez";
-    std::string path = ".";
-    path += CROSS_FILESPLIT;
-    path += dirname;
-    struct stat cstat;
-    int res=stat(path.c_str(),&cstat);
-    if(res==-1 || !(cstat.st_mode & S_IFDIR)) {
-        path = GetDOSBoxXPath();
-        if (path.size()) {
-            path += dirname;
-            res=stat(path.c_str(),&cstat);
-        }
-        if(!path.size() || res==-1 || (cstat.st_mode & S_IFDIR) == 0) {
-            path = "";
-            Cross::CreatePlatformConfigDir(path);
-            path += dirname;
-            res=stat(path.c_str(),&cstat);
-            if(res==-1 || (cstat.st_mode & S_IFDIR) == 0)
-                path = "";
-        }
-    }
-
-    drivezRegister(path, "/");
-
-	VFILE_RegisterBuiltinFileBlob(bfb_EDLIN_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_DEBUG_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_MOVE_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_FIND_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_FCBS_COM, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_FILES_COM, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_LASTDRIV_COM, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_REPLACE_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_SORT_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_XCOPY_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_APPEND_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_DEVICE_COM, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_BUFFERS_COM, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_CHKDSK_EXE, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_COMP_COM, "/DOS/");
-	VFILE_RegisterBuiltinFileBlob(bfb_FC_EXE, "/DOS/");
-#if C_IPX
-	if (addipx) PROGRAMS_MakeFile("IPXNET.COM",IPXNET_ProgramStart,"/SYSTEM/");
-#endif
-	if (addne2k) VFILE_RegisterBuiltinFileBlob(bfb_NE2000_COM, "/SYSTEM/");
-	if (addovl) VFILE_RegisterBuiltinFileBlob(bfb_GLIDE2X_OVL, "/SYSTEM/");
-
-	/* These are IBM PC/XT/AT ONLY. They will not work in PC-98 mode. */
-	if (!IS_PC98_ARCH) {
-		VFILE_RegisterBuiltinFileBlob(bfb_SYS_COM, "/DOS/"); /* may rely on INT 13h or IBM PC specific functions and layout */
-		VFILE_RegisterBuiltinFileBlob(bfb_FDISK_EXE, "/DOS/"); /* relies on IBM PC INT 13h */
-		VFILE_RegisterBuiltinFileBlob(bfb_FORMAT_EXE, "/DOS/"); /* does not work in PC-98 mode */
-		VFILE_RegisterBuiltinFileBlob(bfb_DEFRAG_EXE, "/DOS/"); /* relies on IBM PC CGA/EGA/VGA alphanumeric display memory */
-		VFILE_RegisterBuiltinFileBlob(bfb_HEXMEM16_EXE, "/DEBUG/");
-		VFILE_RegisterBuiltinFileBlob(bfb_HEXMEM32_EXE, "/DEBUG/");
-		VFILE_RegisterBuiltinFileBlob(bfb_DOSIDLE_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_DOS32A_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_DOS4GW_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_CDPLAY_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_CDPLAY_TXT, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_CDPLAY_ZIP, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_DOSMID_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_MPXPLAY_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_ZIP_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_UNZIP_EXE, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_EMSMAGIC_COM, "/BIN/");
-		VFILE_RegisterBuiltinFileBlob(bfb_DISKCOPY_EXE, "/DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_PRINT_COM, "/DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_EDIT_COM, "/DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_LICENSE_TXT, "/4DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_EXAMPLES_BTM, "/4DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_BATCOMP_EXE, "/4DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_OPTION_EXE, "/4DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_4HELP_EXE, "/4DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_4DOS_HLP, "/4DOS/");
-		VFILE_RegisterBuiltinFileBlob(bfb_4DOS_COM, "/4DOS/");
-	}
-
-	if (IS_VGA_ARCH) {
-        VFILE_RegisterBuiltinFileBlob(bfb_VGA_COM, "/TEXTUTIL/");
-        VFILE_RegisterBuiltinFileBlob(bfb_SCANRES_COM, "/TEXTUTIL/");
-        VFILE_RegisterBuiltinFileBlob(bfb_EGA_COM, "/TEXTUTIL/");
-        VFILE_RegisterBuiltinFileBlob(bfb_CLR_COM, "/TEXTUTIL/");
-        VFILE_RegisterBuiltinFileBlob(bfb_CGA_COM, "/TEXTUTIL/");
-        VFILE_RegisterBuiltinFileBlob(bfb_50_COM, "/TEXTUTIL/");
-        VFILE_RegisterBuiltinFileBlob(bfb_28_COM, "/TEXTUTIL/");
-    } else if (IS_EGA_ARCH)
-        VFILE_RegisterBuiltinFileBlob(bfb_28_COM_ega, "/TEXTUTIL/");
-
-    if (IS_VGA_ARCH)
-        VFILE_RegisterBuiltinFileBlob(bfb_25_COM, "/TEXTUTIL/");
-    else if (IS_EGA_ARCH)
-        VFILE_RegisterBuiltinFileBlob(bfb_25_COM_ega, "/TEXTUTIL/");
-    else if (!IS_PC98_ARCH)
-        VFILE_RegisterBuiltinFileBlob(bfb_25_COM_other, "/TEXTUTIL/");
-
-    /* MEM.COM is not compatible with PC-98 and/or 8086 emulation */
-    if(!IS_PC98_ARCH && CPU_ArchitectureType >= CPU_ARCHTYPE_80186)
-        VFILE_RegisterBuiltinFileBlob(bfb_MEM_EXE, "/DOS/");
-
-    VFILE_RegisterBuiltinFileBlob(bfb_CWSDPMI_EXE, "/BIN/");
-    /* DSXMENU.EXE */
-    if(IS_PC98_ARCH)
-        VFILE_RegisterBuiltinFileBlob(bfb_DSXMENU_EXE_PC98, "/BIN/");
-    else {
-        VFILE_RegisterBuiltinFileBlob(bfb_DSXMENU_EXE_PC, "/BIN/");
-        VFILE_RegisterBuiltinFileBlob(bfb_SHUTDOWN_COM, "/BIN/");
-    }
-
-	VFILE_RegisterBuiltinFileBlob(bfb_EVAL_EXE, "/BIN/");
-    if(!IS_PC98_ARCH)
-        VFILE_RegisterBuiltinFileBlob(bfb_EVAL_HLP, "/BIN/");
-
-
 	DOS_PSP psp(psp_seg);
 	psp.MakeNew(0);
 	dos.psp(psp_seg);
@@ -1839,6 +1752,7 @@ void SHELL_Run() {
     char namestr[CROSS_LEN], tmpstr[CROSS_LEN], *name=namestr, *tmp=tmpstr;
     SHELL_ProgramStart_First_shell(&first_shell);
     first_shell->Prepare();
+    prepared = true;
     if (section!=NULL&&!control->opt_noconfig&&!control->opt_securemode&&!control->SecureMode()) {
         char *shell = (char *)section->Get_string("shell");
         if (strlen(shell)) {
@@ -1877,6 +1791,7 @@ void SHELL_Run() {
 		first_shell->Run();
 		delete first_shell;
 		first_shell = 0;//Make clear that it shouldn't be used anymore
+		prepared = false;
 		dos_shell_running_program = false;
 #if defined(WIN32) && !defined(C_SDL2)
 		int Reflect_Menu(void);
@@ -1886,6 +1801,7 @@ void SHELL_Run() {
 	catch (...) {
 		delete first_shell;
 		first_shell = 0;//Make clear that it shouldn't be used anymore
+		prepared = false;
 		dos_shell_running_program = false;
 #if defined(WIN32) && !defined(C_SDL2)
 		int Reflect_Menu(void);
